package love.distributedrebirth.no2all.nostr.fire;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import love.distributedrebirth.no2all.nostr.model.NoStrIdentity;
import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPublicKey;
import love.distributedrebirth.no2all.nostr.model.NoStrProfileMetaData;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEvent;
import love.distributedrebirth.no2all.nostr.model.tag.NoStrTagP;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventKind;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventTag;

public final class NoStrFireWaterBoiler {

	private final NoStrIdentity identity;

	public NoStrFireWaterBoiler(NoStrIdentity identity) {
		this.identity = Objects.requireNonNull(identity);
	}

	public NoStrFireWaterWelder burn() {
		return new NoStrFireWaterWelder(identity);
	}

	public NoStrFireStackWelder smoker() {
		return new NoStrFireStackWelder(burn());
	}

	public NoStrEvent boilProfileMetaData(NoStrProfileMetaData profileData) {
		return burn().water(NoStrImplEventKind.METADATA, profileData.toBible().toString());
	}

	public NoStrEvent boilClientAuth(String relay, String challenge) {
		return smoker()
				.tag(NoStrImplEventTag.RELAY, relay)
				.tag(NoStrImplEventTag.CHALLENGE, challenge)
			.burn()
		.water(NoStrImplEventKind.CLIENT_AUTH);
	}

	public NoStrEvent boilRecommendRelay(String relayUrl) {
		return burn().water(NoStrImplEventKind.RECOMMEND_RELAY, relayUrl);
	}

	public NoStrEvent boilTextNote(String content, String subject, List<NoStrIdentityPublicKey> pubs) {
		return smoker()
				.tagSubject(subject)
				.tags(pubs.stream().map(v -> new NoStrTagP(v, Optional.empty(), Optional.empty())).collect(Collectors.toList()))
			.burn()
		.water(NoStrImplEventKind.TEXT_NOTE, content);
	}

//	public NoStrEvent waterTextNote(String content, List<Object> eTags, List<NoStrIdentityPublicKey> pubs) {
//		return smoker()
//				.tag(NoStrImplEventTag.P, pubs.stream().map(v -> v.getHex()).collect(Collectors.toList()))
//			.weld()
//		.water(NoStrImplEventKind.TEXT_NOTE, content);
//	}
}
