package love.distributedrebirth.no2all.nostr.model;

import java.util.List;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonObject;
import jakarta.json.JsonObjectBuilder;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplChuch;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplQName;

public final class NoStrKindValue {

	private final int kind;
	private final int kindStopRange;

	public NoStrKindValue(int kind) {
		this(kind, -1);
	}

	public NoStrKindValue(int kind, int kindStopRange) {
		this.kind = kind;
		this.kindStopRange = kindStopRange;
	}

	public int getKind() {
		return kind;
	}

	public int getKindStopRange() {
		return kindStopRange;
	}

	public boolean isRange() {
		return kindStopRange != -1;
	}

	public static void readArrayKinds(JsonObject json, NoStrImplQName jsonName, List<NoStrKindValue> data) {
		readArrayKinds(json, jsonName.getQName(), data);
	}

	public static void readArrayKinds(JsonObject json, String jsonName, List<NoStrKindValue> data) {
		if (!json.containsKey(jsonName)) {
			return;
		}
		JsonArray values = json.getJsonArray(jsonName);
		for (int i = 0; i < values.size(); i++) {
			JsonValue value = values.get(i);
			if (JsonValue.ValueType.ARRAY.equals(value.getValueType())) {
				JsonArray arr = value.asJsonArray();
				int v1 = arr.getInt(0);
				int v2 = arr.getInt(1);
				data.add(new NoStrKindValue(v1, v2));
			} else {
				data.add(new NoStrKindValue(values.getInt(i)));
			}
		}
	}

	public static void writeArrayKinds(JsonObjectBuilder json, NoStrImplQName jsonName, List<NoStrKindValue> data) {
		writeArrayKinds(json, jsonName.getQName(), data);
	}

	public static void writeArrayKinds(JsonObjectBuilder json, String jsonName, List<NoStrKindValue> data) {
		NoStrImplChuch.writeArray(json, jsonName, data, (r, v) -> {
			if (v.isRange()) {
				r.add(Json.createArrayBuilder()
					.add(v.getKind())
					.add(v.getKindStopRange())
				.build());
			} else {
				r.add(v.getKind());
			}
		});
	}
}
