package love.distributedrebirth.no2all.nostr.model;

import java.util.ArrayList;
import java.util.List;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonObject;
import jakarta.json.JsonObjectBuilder;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.NoStrGospel;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplChuch;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplQName;

public final class NoStrRelayInfo implements NoStrGospel {

	private String id;
	private String name;
	private String description;
	private String publicKey;
	private String contact;
	private final List<Integer> supportedNips = new ArrayList<>();
	private String software;
	private String version;

	private NoStrRelayInfoLimitation limitation;
	private final List<NoStrRelayInfoRetention> retentions = new ArrayList<>();

	private final List<String> relayCountries = new ArrayList<>();
	private final List<String> languageTags = new ArrayList<>();
	private final List<String> tags = new ArrayList<>();
	private String postingPolicy;

	private String paymentUrl;
	private final List<NoStrRelayInfoPayment> feesAdmission = new ArrayList<>();
	private final List<NoStrRelayInfoPayment> feesSubscription = new ArrayList<>();
	private final List<NoStrRelayInfoPayment> feesPublication = new ArrayList<>();

	public NoStrRelayInfo() {
	}

	public NoStrRelayInfo(JsonObject json) {
		id = NoStrImplChuch.readString(json, QName.ID);
		name = NoStrImplChuch.readString(json, QName.NAME);
		description = NoStrImplChuch.readString(json, QName.DESCRIPTION);
		publicKey = NoStrImplChuch.readString(json, QName.PUBKEY);
		contact = NoStrImplChuch.readString(json, QName.CONTACT);
		if (json.containsKey(QName.SUPPORTED_NIPS.getQName())) {
			JsonArray values = json.getJsonArray(QName.SUPPORTED_NIPS.getQName());
			for (int i = 0; i < values.size(); i++) {
				supportedNips.add(values.getInt(i));
			}	
		}
		software = NoStrImplChuch.readString(json, QName.SOFTWARE);
		version = NoStrImplChuch.readString(json, QName.VERSION);
		JsonValue limitationJson = json.get(QName.LIMITATION.getQName());
		if (limitationJson != null && JsonValue.ValueType.OBJECT.equals(limitationJson.getValueType())) {
			limitation = new NoStrRelayInfoLimitation(limitationJson.asJsonObject());
		}
		NoStrImplChuch.readArray(json, QName.RETENTION, v -> retentions.add(new NoStrRelayInfoRetention(v.asJsonObject())));
		NoStrImplChuch.readArrayString(json, QName.RELAY_COUNTRIES, relayCountries);
		NoStrImplChuch.readArrayString(json, QName.LANGUAGE_TAGS, languageTags);
		NoStrImplChuch.readArrayString(json, QName.TAGS, tags);
		postingPolicy = NoStrImplChuch.readString(json, QName.POSTING_POLICY);
		paymentUrl = NoStrImplChuch.readString(json, QName.PAYMENT_URL);
		JsonValue feesJson = json.get(QName.FEES.getQName());
		if (feesJson != null && JsonValue.ValueType.OBJECT.equals(feesJson.getValueType())) {
			JsonObject feesObject = feesJson.asJsonObject();
			NoStrImplChuch.readArray(feesObject, QName.ADMISSION, v -> feesAdmission.add(new NoStrRelayInfoPayment(v.asJsonObject())));
			NoStrImplChuch.readArray(feesObject, QName.SUBSCRIPTION, v -> feesSubscription.add(new NoStrRelayInfoPayment(v.asJsonObject())));
			NoStrImplChuch.readArray(feesObject, QName.PUBLICATION, v -> feesPublication.add(new NoStrRelayInfoPayment(v.asJsonObject())));
		}
	}

	@Override
	public JsonValue toBible() {
		JsonObjectBuilder result = Json.createObjectBuilder();
		NoStrImplChuch.writeString(result, QName.ID, id);
		NoStrImplChuch.writeString(result, QName.NAME, name);
		NoStrImplChuch.writeString(result, QName.DESCRIPTION, description);
		NoStrImplChuch.writeString(result, QName.PUBKEY, publicKey);
		NoStrImplChuch.writeString(result, QName.CONTACT, contact);
		NoStrImplChuch.writeArray(result, QName.SUPPORTED_NIPS, supportedNips, (r, v) -> r.add(v));
		NoStrImplChuch.writeString(result, QName.SOFTWARE, software);
		NoStrImplChuch.writeString(result, QName.VERSION, version);
		if (limitation != null) {
			result.add(QName.LIMITATION.getQName(), limitation.toBible());
		}
		NoStrImplChuch.writeArrayWriter(result, QName.RETENTION, retentions);
		NoStrImplChuch.writeArrayString(result, QName.RELAY_COUNTRIES, relayCountries);
		NoStrImplChuch.writeArrayString(result, QName.LANGUAGE_TAGS, languageTags);
		NoStrImplChuch.writeArrayString(result, QName.TAGS, tags);
		NoStrImplChuch.writeString(result, QName.POSTING_POLICY, postingPolicy);
		NoStrImplChuch.writeString(result, QName.PAYMENT_URL, paymentUrl);
		
		JsonObjectBuilder feesObject = Json.createObjectBuilder();
		NoStrImplChuch.writeArrayWriter(feesObject, QName.ADMISSION, feesAdmission);
		NoStrImplChuch.writeArrayWriter(feesObject, QName.SUBSCRIPTION, feesSubscription);
		NoStrImplChuch.writeArrayWriter(feesObject, QName.PUBLICATION, feesPublication);
		result.add(QName.FEES.getQName(), feesObject);
		return result.build();
	}

	private enum QName implements NoStrImplQName {

		ID,
		NAME,
		DESCRIPTION,
		PUBKEY,
		CONTACT,
		SUPPORTED_NIPS,
		SOFTWARE,
		VERSION,
		LIMITATION,
		RETENTION,
		RELAY_COUNTRIES,
		LANGUAGE_TAGS,
		TAGS,
		POSTING_POLICY,
		PAYMENT_URL,
		ADMISSION,
		SUBSCRIPTION,
		PUBLICATION,
		FEES,
		;

		private final String jsonQName;

		private QName() {
			jsonQName = name().toLowerCase();
		}

		@Override
		public String getQName() {
			return jsonQName;
		}
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getPublicKey() {
		return publicKey;
	}

	public void setPublicKey(String publicKey) {
		this.publicKey = publicKey;
	}

	public String getContact() {
		return contact;
	}

	public void setContact(String contact) {
		this.contact = contact;
	}

	public String getSoftware() {
		return software;
	}

	public void setSoftware(String software) {
		this.software = software;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public NoStrRelayInfoLimitation getLimitation() {
		return limitation;
	}

	public void setLimitation(NoStrRelayInfoLimitation limitation) {
		this.limitation = limitation;
	}

	public String getPostingPolicy() {
		return postingPolicy;
	}

	public void setPostingPolicy(String postingPolicy) {
		this.postingPolicy = postingPolicy;
	}

	public String getPaymentUrl() {
		return paymentUrl;
	}

	public void setPaymentUrl(String paymentUrl) {
		this.paymentUrl = paymentUrl;
	}

	public List<Integer> getSupportedNips() {
		return supportedNips;
	}

	public List<NoStrRelayInfoRetention> getRetentions() {
		return retentions;
	}

	public List<String> getRelayCountries() {
		return relayCountries;
	}

	public List<String> getLanguageTags() {
		return languageTags;
	}

	public List<String> getTags() {
		return tags;
	}

	public List<NoStrRelayInfoPayment> getFeesAdmission() {
		return feesAdmission;
	}

	public List<NoStrRelayInfoPayment> getFeesSubscription() {
		return feesSubscription;
	}

	public List<NoStrRelayInfoPayment> getFeesPublication() {
		return feesPublication;
	}
}
