package love.distributedrebirth.no2all.nostr.model;

import jakarta.json.Json;
import jakarta.json.JsonObject;
import jakarta.json.JsonObjectBuilder;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.NoStrGospel;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplChuch;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplQName;

public final class NoStrRelayInfoLimitation implements NoStrGospel {

	private Integer maxMessageLength;
	private Integer maxSubscriptions;
	private Integer maxFilters;
	private Integer maxLimit;
	private Integer maxSubIdLength;
	private Integer minPrefix;
	private Integer maxEventTags;
	private Integer maxContentLength;
	private Integer minPowDifficulty;
	private Boolean authRequired;
	private Boolean paymentRequired;

	public NoStrRelayInfoLimitation() {
	}

	public NoStrRelayInfoLimitation(JsonObject json) {
		maxMessageLength = NoStrImplChuch.readInteger(json, QName.MAX_MESSAGE_LENGTH);
		maxSubscriptions = NoStrImplChuch.readInteger(json, QName.MAX_SUBSCRIPTIONS);
		maxFilters = NoStrImplChuch.readInteger(json, QName.MAX_FILTERS);
		maxLimit = NoStrImplChuch.readInteger(json, QName.MAX_LIMIT);
		maxSubIdLength = NoStrImplChuch.readInteger(json, QName.MAX_SUBID_LENGTH);
		minPrefix = NoStrImplChuch.readInteger(json, QName.MIN_PREFIX);
		maxEventTags = NoStrImplChuch.readInteger(json, QName.MAX_EVENT_TAGS);
		maxContentLength = NoStrImplChuch.readInteger(json, QName.MAX_CONTENT_LENGTH);
		minPowDifficulty = NoStrImplChuch.readInteger(json, QName.MIN_POW_DIFFICULTY);
		authRequired = NoStrImplChuch.readBoolean(json, QName.AUTH_REQUIRED);
		paymentRequired = NoStrImplChuch.readBoolean(json, QName.PAYMENT_REQUIRED);
	}

	@Override
	public JsonValue toBible() {
		JsonObjectBuilder result = Json.createObjectBuilder();
		NoStrImplChuch.writeInteger(result, QName.MAX_MESSAGE_LENGTH, maxMessageLength);
		NoStrImplChuch.writeInteger(result, QName.MAX_SUBSCRIPTIONS, maxSubscriptions);
		NoStrImplChuch.writeInteger(result, QName.MAX_FILTERS, maxFilters);
		NoStrImplChuch.writeInteger(result, QName.MAX_LIMIT, maxLimit);
		NoStrImplChuch.writeInteger(result, QName.MAX_SUBID_LENGTH, maxSubIdLength);
		NoStrImplChuch.writeInteger(result, QName.MIN_PREFIX, minPrefix);
		NoStrImplChuch.writeInteger(result, QName.MAX_EVENT_TAGS, maxEventTags);
		NoStrImplChuch.writeInteger(result, QName.MAX_CONTENT_LENGTH, maxContentLength);
		NoStrImplChuch.writeInteger(result, QName.MIN_POW_DIFFICULTY, minPowDifficulty);
		NoStrImplChuch.writeBoolean(result, QName.AUTH_REQUIRED, authRequired);
		NoStrImplChuch.writeBoolean(result, QName.PAYMENT_REQUIRED, paymentRequired);
		return result.build();
	}
	
	private enum QName implements NoStrImplQName {

		MAX_MESSAGE_LENGTH,
		MAX_SUBSCRIPTIONS,
		MAX_FILTERS,
		MAX_LIMIT,
		MAX_SUBID_LENGTH,
		MIN_PREFIX,
		MAX_EVENT_TAGS,
		MAX_CONTENT_LENGTH,
		MIN_POW_DIFFICULTY,
		AUTH_REQUIRED,
		PAYMENT_REQUIRED,
		;

		private final String jsonQName;

		private QName() {
			jsonQName = name().toLowerCase();
		}

		@Override
		public String getQName() {
			return jsonQName;
		}
	}

	public Integer getMaxMessageLength() {
		return maxMessageLength;
	}

	public void setMaxMessageLength(Integer maxMessageLength) {
		this.maxMessageLength = maxMessageLength;
	}

	public Integer getMaxSubscriptions() {
		return maxSubscriptions;
	}

	public void setMaxSubscriptions(Integer maxSubscriptions) {
		this.maxSubscriptions = maxSubscriptions;
	}

	public Integer getMaxFilters() {
		return maxFilters;
	}

	public void setMaxFilters(Integer maxFilters) {
		this.maxFilters = maxFilters;
	}

	public Integer getMaxLimit() {
		return maxLimit;
	}

	public void setMaxLimit(Integer maxLimit) {
		this.maxLimit = maxLimit;
	}

	public Integer getMaxSubIdLength() {
		return maxSubIdLength;
	}

	public void setMaxSubIdLength(Integer maxSubIdLength) {
		this.maxSubIdLength = maxSubIdLength;
	}

	public Integer getMinPrefix() {
		return minPrefix;
	}

	public void setMinPrefix(Integer minPrefix) {
		this.minPrefix = minPrefix;
	}

	public Integer getMaxEventTags() {
		return maxEventTags;
	}

	public void setMaxEventTags(Integer maxEventTags) {
		this.maxEventTags = maxEventTags;
	}

	public Integer getMaxContentLength() {
		return maxContentLength;
	}

	public void setMaxContentLength(Integer maxContentLength) {
		this.maxContentLength = maxContentLength;
	}

	public Integer getMinPowDifficulty() {
		return minPowDifficulty;
	}

	public void setMinPowDifficulty(Integer minPowDifficulty) {
		this.minPowDifficulty = minPowDifficulty;
	}

	public Boolean getAuthRequired() {
		return authRequired;
	}

	public void setAuthRequired(Boolean authRequired) {
		this.authRequired = authRequired;
	}

	public Boolean getPaymentRequired() {
		return paymentRequired;
	}

	public void setPaymentRequired(Boolean paymentRequired) {
		this.paymentRequired = paymentRequired;
	}
}
