package love.distributedrebirth.no2all.nostr.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import jakarta.json.Json;
import jakarta.json.JsonObject;
import jakarta.json.JsonObjectBuilder;
import jakarta.json.JsonString;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.NoStrGospel;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventTagMeta;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplChuch;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplQName;

public class NoStrSubscriptionFilter implements NoStrGospel {

	private final static String TAG_FILTER_PREFIX = "#";
	private final List<String> ids = new ArrayList<>();
	private final List<String> authors = new ArrayList<>();
	private final List<NoStrKindValue> kinds = new ArrayList<>();
	private final Map<String, NoStrSubscriptionFilterTag> tags = new HashMap<>();
	private Integer since;
	private Integer until;
	private Integer limit;

	private enum QName implements NoStrImplQName {
		IDS, AUTHORS, KINDS, SINCE, UNTIL, LIMIT;
		
		private final String jsonQName;

		private QName() {
			jsonQName = name().toLowerCase();
		}

		@Override
		public String getQName() {
			return jsonQName;
		}
	}

	public NoStrSubscriptionFilter() {
	}

	public NoStrSubscriptionFilter(JsonObject json) {
		NoStrImplChuch.readArrayString(json, QName.IDS, ids);
		NoStrImplChuch.readArrayString(json, QName.AUTHORS, authors);
		NoStrKindValue.readArrayKinds(json, QName.KINDS, kinds);
		for (String jsonName : json.keySet()) {
			if (!jsonName.startsWith(TAG_FILTER_PREFIX)) {
				continue;
			}
			String jsonNameKey = jsonName.substring(0, 1);
			if (tags.containsKey(jsonNameKey)) {
				continue;
			}
			JsonValue arrayJson = json.get(jsonNameKey);
			if (!JsonValue.ValueType.ARRAY.equals(arrayJson.getValueType())) {
				continue;
			}
			Stream<JsonValue> tagFilterValues = arrayJson.asJsonArray().stream();
			List<String> tagFilters = tagFilterValues.map(v -> ((JsonString)v).getString()).collect(Collectors.toList());
			NoStrEventTagMeta eventTag = NoStrEventTagMeta.valueOf(jsonNameKey);
			tags.put(jsonNameKey, new NoStrSubscriptionFilterTag(eventTag, tagFilters));
		}
		since = NoStrImplChuch.readInteger(json, QName.SINCE);
		until = NoStrImplChuch.readInteger(json, QName.UNTIL);
		limit = NoStrImplChuch.readInteger(json, QName.LIMIT);
	}

	@Override
	public JsonValue toBible() {
		JsonObjectBuilder result = Json.createObjectBuilder();
		NoStrImplChuch.writeArrayString(result, QName.IDS, ids);
		NoStrImplChuch.writeArrayString(result, QName.AUTHORS, authors);
		NoStrKindValue.writeArrayKinds(result, QName.KINDS, kinds);
		for (String tagKey : tags.keySet()) {
			if (tagKey.length() != 1) {
				continue;
			}
			NoStrSubscriptionFilterTag tag = tags.get(tagKey);
			result.add(TAG_FILTER_PREFIX + tagKey, Json.createArrayBuilder(tag.getKeywords()));
		}
		NoStrImplChuch.writeInteger(result, QName.SINCE, since);
		NoStrImplChuch.writeInteger(result, QName.UNTIL, until);
		NoStrImplChuch.writeInteger(result, QName.LIMIT, limit);
		return result.build();
	}

	public Integer getSince() {
		return since;
	}

	public void setSince(Integer since) {
		this.since = since;
	}

	public Integer getUntil() {
		return until;
	}

	public void setUntil(Integer until) {
		this.until = until;
	}

	public Integer getLimit() {
		return limit;
	}

	public void setLimit(Integer limit) {
		this.limit = limit;
	}

	public List<String> getIds() {
		return ids;
	}

	public List<String> getAuthors() {
		return authors;
	}

	public List<NoStrKindValue> getKinds() {
		return kinds;
	}

	public Map<String, NoStrSubscriptionFilterTag> getTags() {
		return tags;
	}
}
