package love.distributedrebirth.no2all.nostr.model.event;

import java.time.Instant;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonObject;
import jakarta.json.JsonString;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.NoStrGospel;
import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPublicKey;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplQName;

public final class NoStrEvent implements NoStrGospel {

	private final NoStrEventId id;
	private final NoStrEventPayload payload;
	private final NoStrEventSignature signature;

	private enum QName implements NoStrImplQName {

		ID, PUBKEY, CREATED_AT, KIND, TAGS, CONTENT, SIG;

		private final String jsonQName;

		private QName() {
			jsonQName = name().toLowerCase();
		}

		@Override
		public String getQName() {
			return jsonQName;
		}
	}

	public NoStrEvent(NoStrEventId id, NoStrEventPayload payload, NoStrEventSignature signature) {
		this.id = Objects.requireNonNull(id);
		this.payload = Objects.requireNonNull(payload);
		this.signature = Objects.requireNonNull(signature);
	}

	public NoStrEvent(JsonObject event) {
		List<NoStrEventTag> eventTags = new ArrayList<>();
		JsonArray jsonTags = event.getJsonArray(QName.TAGS.getQName());
		for (JsonValue jsonTag : jsonTags) {
			Iterator<JsonValue> i = jsonTag.asJsonArray().iterator();
			if (!i.hasNext()) {
				throw new IllegalArgumentException("Empty tag");
			}
			String tagStr = ((JsonString)i.next()).getString();
			NoStrEventTagMeta tagMeta = NoStrEventTagMeta.valueOf(tagStr);
			List<String> args = new ArrayList<>();
			while (i.hasNext()) {
				args.add(((JsonString)i.next()).getString());
			}
			eventTags.add(NoStrEventTag.valueOf(tagMeta, args.toArray(new String[] {})));
		}
		long createdAtLong = event.getJsonNumber(QName.CREATED_AT.getQName()).longValue();
		int kindNumber = event.getInt(QName.KIND.getQName());
		String idStr = event.getString(QName.ID.getQName());
		String signatureStr = event.getString(QName.SIG.getQName());
		String pubkeyStr = event.getString(QName.PUBKEY.getQName());
		String content = event.getString(QName.CONTENT.getQName());
		NoStrIdentityPublicKey pubkey = NoStrIdentityPublicKey.ofHex(pubkeyStr);
		Instant createdAt = Instant.ofEpochSecond(createdAtLong);
		NoStrEventKind kind = NoStrEventKind.valueOf(kindNumber);
		this.id = NoStrEventId.ofHex(idStr);
		this.payload = new NoStrEventPayload(pubkey, createdAt, kind, eventTags, content);
		this.signature = NoStrEventSignature.ofHex(signatureStr);
	}

	@Override
	public JsonValue toBible() {
		return Json.createObjectBuilder()
			.add(QName.ID.getQName(), id.getHex())
			.add(QName.PUBKEY.getQName(), payload.getPublicKey().getHex())
			.add(QName.CREATED_AT.getQName(), payload.getCreatedAt().getEpochSecond())
			.add(QName.KIND.getQName(), payload.getKind().getNumber())
			.add(QName.TAGS.getQName(), NoStrEventTag.toJsonArray(payload.getTags()))
			.add(QName.CONTENT.getQName(), payload.getContent())
			.add(QName.SIG.getQName(), signature.getHex())
		.build();
	}

	public NoStrEventId getId() {
		return id;
	}

	public NoStrEventPayload getPayload() {
		return payload;
	}

	public NoStrEventSignature getSignature() {
		return signature;
	}
}
