package love.distributedrebirth.no2all.nostr.model.event;

import java.time.Instant;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPublicKey;

public final class NoStrEventPayload {

	private final NoStrIdentityPublicKey publicKey;
	private Instant createdAt;
	private final NoStrEventKind kind;
	private final List<NoStrEventTag> tags;
	private final String content;

	public NoStrEventPayload(NoStrIdentityPublicKey publicKey, Instant createdAt, NoStrEventKind kind, List<NoStrEventTag> tags,
			String content) {
		this.publicKey = Objects.requireNonNull(publicKey);
		this.createdAt = Objects.requireNonNull(createdAt);
		this.kind = Objects.requireNonNull(kind);
		this.tags = Objects.requireNonNull(tags);
		this.content = Objects.requireNonNull(content);
	}

	public NoStrIdentityPublicKey getPublicKey() {
		return publicKey;
	}

	public Instant getCreatedAt() {
		return createdAt;
	}

	public NoStrEventKind getKind() {
		return kind;
	}

	public List<NoStrEventTag> getTags() {
		return tags;
	}

	public String getContent() {
		return content;
	}

	public void setCreatedAt(Instant createdAt) {
		this.createdAt = Objects.requireNonNull(createdAt);
	}

	public List<NoStrEventTag> findByQName(String tagQName) {
		List<NoStrEventTag> result = new ArrayList<>();
		for (int i=0; i < tags.size(); i++) {
			NoStrEventTag tag = tags.get(i);
			if (tag.getMeta().getQName().equals(tagQName)) {
				result.add(tag);
			}
		}
		return result;
	}

	public List<NoStrEventTag> findByQName(NoStrEventTagMeta tagQName) {
		return findByQName(tagQName.getQName());
	}

	public Optional<NoStrEventTag> findFirstByQName(String tagQName) {
		for (int i=0; i < tags.size(); i++) {
			NoStrEventTag tag = tags.get(i);
			if (tag.getMeta().getQName().equals(tagQName)) {
				return Optional.of(tag);
			}
		}
		return Optional.empty();
	}

	public Optional<NoStrEventTag> findFirstByQName(NoStrEventTagMeta tagQName) {
		return findFirstByQName(tagQName.getQName());
	}
}
