package love.distributedrebirth.no2all.nostr.model.event;

import java.io.StringWriter;
import java.nio.charset.StandardCharsets;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonWriter;
import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPrivateKey;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplBinHexAdapter;
import love.distributedrebirth.no2all.octo.OctoBitFormat;
import love.distributedrebirth.no2all.octo.trust.OctoTrust;
import love.distributedrebirth.no2all.octo.trust.OctoTrustHash;
import love.distributedrebirth.no2all.octo.trust.OctoTrustSchnorr;

public final class NoStrEventSignature extends NoStrImplBinHexAdapter {

	public NoStrEventSignature(byte[] valueBytes) {
		super(valueBytes);
	}

	public static NoStrEventSignature ofHex(String valueHex) {
		return new NoStrEventSignature(OctoBitFormat.HEX.toBytes(valueHex));
	}

	public static NoStrEvent sign(NoStrEventPayload payload, NoStrIdentityPrivateKey privateKey, byte[] eventId) {
		byte[] signRandom = OctoTrust.createRandomByteArray(OctoTrust.KEY_LENGTH);
		byte[] signHash = OctoTrustSchnorr.sign(eventId, privateKey.getBytes(), signRandom);
		NoStrEventSignature signature = new NoStrEventSignature(signHash);
		return new NoStrEvent(new NoStrEventId(eventId), payload, signature);
	}

	public static NoStrEvent sign(NoStrEventPayload payload, NoStrIdentityPrivateKey privateKey) {
		return sign(payload, privateKey, generateEventId(payload));
	}

	public static boolean verify(NoStrEvent event) {
		byte[] eventId = generateEventId(event.getPayload());
		byte[] pubkey = event.getPayload().getPublicKey().getBytes();
		byte[] sig = event.getSignature().getBytes();
		boolean result = OctoTrustSchnorr.verify(eventId, pubkey, sig);
		return result;
	}

	public static String generateEventIdJsonString(NoStrEventPayload payload) {
		JsonArray eventAsJson = Json.createArrayBuilder()
			.add(0)
			.add(payload.getPublicKey().getHex())
			.add(payload.getCreatedAt().getEpochSecond())
			.add(payload.getKind().getNumber())
			.add(NoStrEventTag.toJsonArray(payload.getTags()))
			.add(payload.getContent())
			.build();
		StringWriter eventAsString = new StringWriter();
		try (JsonWriter writer = Json.createWriter(eventAsString)) {
			writer.write(eventAsJson);
		}
		return eventAsString.getBuffer().toString();
	}

	public static byte[] generateEventIdHash(String eventIdJsonString) {
		return OctoTrustHash.sha256(eventIdJsonString.getBytes(StandardCharsets.UTF_8));
	}

	public static byte[] generateEventId(NoStrEventPayload payload) {
		return generateEventIdHash(generateEventIdJsonString(payload));
	}
}
