package love.distributedrebirth.no2all.nostr.model.event;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.NoStrGospel;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventTag;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventTagArgument;

public interface NoStrEventTag extends NoStrGospel {

	NoStrEventTagMeta getMeta();

	String[] getMetaArguments();

	default Optional<String> getMetaArgument(int idx) {
		if (idx >= getMetaArguments().length || idx < 0) {
			return Optional.empty();
		}
		return Optional.ofNullable(getMetaArguments()[idx]);
	}

	default Optional<String> getMetaArgument(NoStrImplEventTagArgument idx) {
		return getMetaArgument(idx.ordinal());
	}

	default List<String> getMetaSafeArguments() {
		List<String> result = new ArrayList<>();
		int idxLastValue = 0;
		String[] values = getMetaArguments();
		for (int i = 0; i < values.length; i++) {
			String str = values[i];
			if (str == null) {
				result.add(""); // optional
			} else {
				result.add(str);
				idxLastValue = i;
			}
		}
		if (idxLastValue < result.size()) {
			result = result.subList(0, idxLastValue + 1);
		}
		return result;
	}

	@Override
	default JsonValue toBible() {
		return Json.createArrayBuilder()
				.add(getMeta().getQName())
				.addAll(Json.createArrayBuilder(getMetaSafeArguments()))
			.build();
	}

	static JsonArray toJsonArray(List<NoStrEventTag> tags) {
		return Json.createArrayBuilder(
				tags.stream().map(v -> v.toBible()).collect(Collectors.toList())
			).build();
	}

	static NoStrEventTag valueOfArgs(NoStrEventTagMeta meta, String... args) {
		return valueOf(meta, args);
	}

	static NoStrEventTag valueOf(NoStrEventTagMeta meta, String[] args) {
		Objects.requireNonNull(meta);
		Objects.requireNonNull(args);
		if (meta instanceof NoStrImplEventTag) {
			NoStrImplEventTag metaTag = NoStrImplEventTag.class.cast(meta);
			if (metaTag.getMapperToEventTag().isPresent()) {
				return metaTag.getMapperToEventTag().get().apply(args);
			}
		}
		return new NoStrEventTagCustom(meta, args);
	}
}
