package love.distributedrebirth.no2all.nostr.model.event;

import java.util.Objects;
import java.util.Optional;

import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventTagArgument;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplQName;

public class NoStrEventTagBase<T extends NoStrImplEventTagArgument> implements NoStrEventTag {

	private final NoStrEventTagMeta meta;
	private String[] args;

	public NoStrEventTagBase(NoStrEventTagMeta meta) {
		this(meta, new String[4]); // limit autoGrow to zero in 99% of tags
	}

	public NoStrEventTagBase(NoStrEventTagMeta meta, String[] args) {
		this.meta = Objects.requireNonNull(meta);
		this.args = Objects.requireNonNull(args);
	}

	@Override
	public NoStrEventTagMeta getMeta() {
		return meta;
	}

	@Override
	public String[] getMetaArguments() {
		return args;
	}

	private int autoGrow(int index) {
		if (index < args.length) {
			return index;
		}
		String[] result = new String[index];
		System.arraycopy(args, 0, result, 0, args.length);
		args = result;
		return index;
	}

//	public Optional<String> getMetaArgument(T arg) {
//		return NoStrEventTag.super.getMetaArgument(arg.ordinal());
//	}

	protected void setMetaArgumentStr(T arg, String value) {
		int index = autoGrow(arg.ordinal());
		args[index] = value;
	}

	protected void setMetaArgumentOptStr(T arg, Optional<String> value) {
		if (value.isEmpty()) {
			return;
		}
		setMetaArgumentStr(arg, value.get());
	}

	protected void setMetaArgumentOptQName(T arg, Optional<? extends NoStrImplQName> value) {
		if (value.isEmpty()) {
			return;
		}
		setMetaArgumentStr(arg, value.get().getQName());
	}
}
