package love.distributedrebirth.no2all.nostr.model.message;

import java.util.Objects;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonString;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventId;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplMessageOkReason;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplMessageType;

public final class NoStrMsgToClientOk implements NoStrMsgToClient {

	private final NoStrEventId eventId;
	private final boolean saved;
	private final String message;
	private final NoStrImplMessageOkReason reason;

	public NoStrMsgToClientOk(NoStrEventId eventId, boolean saved, NoStrImplMessageOkReason reason, String message) {
		this.eventId = Objects.requireNonNull(eventId);
		this.saved = saved;
		this.reason = Objects.requireNonNull(reason);
		this.message = reason.getPrefix() + " " + message;
	}

	public NoStrMsgToClientOk(NoStrEventId eventId, boolean saved, String message) {
		this.eventId = Objects.requireNonNull(eventId);
		this.saved = saved;
		this.message = Objects.requireNonNull(message);
		this.reason = NoStrImplMessageOkReason.valueOfPrefix(message);
	}

	public NoStrMsgToClientOk(JsonArray json) {
		this.eventId = NoStrEventId.ofHex(((JsonString)json.get(1)).getString());
		this.saved = JsonValue.TRUE.equals(json.get(2));
		this.message = ((JsonString)json.get(3)).getString();
		this.reason = NoStrImplMessageOkReason.valueOfPrefix(message);
	}

	@Override
	public JsonValue toBible() {
		return Json.createArrayBuilder()
				.add(NoStrImplMessageType.OK.getQName())
				.add(eventId.getHex())
				.add(saved)
				.add(message)
			.build();
	}

	public NoStrEventId getEventId() {
		return eventId;
	}

	public boolean isSaved() {
		return saved;
	}

	public String getMessage() {
		return message;
	}

	public NoStrImplMessageOkReason getReason() {
		return reason;
	}
}
