package love.distributedrebirth.no2all.nostr.model.message;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonString;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.model.NoStrSubscriptionFilter;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplMessageType;

public final class NoStrMsgToRelayReq implements NoStrMsgToRelay {

	private final String subscriptionId;
	private final List<NoStrSubscriptionFilter> filters = new ArrayList<>();

	public NoStrMsgToRelayReq(String subscriptionId, NoStrSubscriptionFilter... filters) {
		this.subscriptionId = Objects.requireNonNull(subscriptionId);
		this.filters.addAll(Arrays.asList(filters));
	}

	public NoStrMsgToRelayReq(JsonArray json) {
		this.subscriptionId = ((JsonString)json.get(1)).getString();
		for (int i = 2; i < json.size(); i++) {
			filters.add(new NoStrSubscriptionFilter(json.get(i).asJsonObject()));
		}
	}

	@Override
	public JsonValue toBible() {
		return Json.createArrayBuilder()
				.add(NoStrImplMessageType.REQ.getQName())
				.add(subscriptionId)
				.addAll(Json.createArrayBuilder(filters.stream().map(v -> v.toBible()).collect(Collectors.toList())))
			.build();
	}

	public String getSubscriptionId() {
		return subscriptionId;
	}

	public List<NoStrSubscriptionFilter> getFilters() {
		return filters;
	}
}
