package love.distributedrebirth.no2all.nostr.model.share;

import java.util.Optional;
import java.util.function.Consumer;

import love.distributedrebirth.no2all.nostr.nip.NoStrImplShareLink;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplShareTLV;
import love.distributedrebirth.no2all.octo.OctoBech32;
import love.distributedrebirth.no2all.octo.OctoBech32Bucket;
import love.distributedrebirth.no2all.octo.OctoBech32String;
import love.distributedrebirth.no2all.octo.OctoBitChain;
import love.distributedrebirth.no2all.octo.OctoBitFormat;

public abstract class NoStrShareB32LinkTLV implements NoStrShareLink {

	private final NoStrShareBitChainTapeDeck tapeDeck;
	private final String shareLink;

	public NoStrShareB32LinkTLV(NoStrImplShareLink linkType, OctoBech32String link) {
		OctoBech32Bucket shareB32 = OctoBech32.fromBech32(linkType.validate(link));
		OctoBitChain chain = new OctoBitChain(shareB32.getData());
		this.tapeDeck = new NoStrShareBitChainTapeDeck(chain);
		this.shareLink = createShareLink(chain, linkType);
	}

	public NoStrShareB32LinkTLV(NoStrImplShareLink linkType, Consumer<NoStrShareBitChainTapeStudio> tapeStudio) {
		OctoBitChain chain = new OctoBitChain();
		tapeStudio.accept(new NoStrShareBitChainTapeStudio(chain));
		this.tapeDeck = new NoStrShareBitChainTapeDeck(chain);
		this.shareLink = createShareLink(chain, linkType);
	}

	private static String createShareLink(OctoBitChain chain, NoStrImplShareLink linkType) {
		return OctoBech32.toBech32(linkType.getPrefix(), chain.toByteArray()).getValue();
	}

	protected NoStrShareBitChainTapeDeck getTapeDeck() {
		return tapeDeck;
	}

	public Optional<String> getClient() {
		return getTapeDeck().playBitFormatOf(NoStrImplShareTLV.CLIENT, OctoBitFormat.UTF8);
	}

	@Override
	public final String toShareLink() {
		return shareLink;
	}
}
