package love.distributedrebirth.no2all.nostr.model.share;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPublicKey;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventKind;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplShareTLV;
import love.distributedrebirth.no2all.octo.OctoBitChain;
import love.distributedrebirth.no2all.octo.OctoBitChainFrame;
import love.distributedrebirth.no2all.octo.OctoBitConverter;
import love.distributedrebirth.no2all.octo.OctoBitFormat;

public class NoStrShareBitChainTapeDeck {

	protected final OctoBitChain chain;

	public NoStrShareBitChainTapeDeck(OctoBitChain chain) {
		this.chain = Objects.requireNonNull(chain);
	}

	public int getFrameCount() {
		return chain.getFrameCount();
	}

	public Optional<ByteBuffer> playByteBufferOf(NoStrImplShareTLV tlv) {
		byte[] data = chain.getFirstDataOfType(tlv);
		if (data == null) {
			return Optional.empty();
		}
		if (data.length == 0) {
			return Optional.empty();
		}
		return Optional.of(ByteBuffer.wrap(data));
	}

	public Optional<ByteBuffer> playSpecial() {
		return playByteBufferOf(NoStrImplShareTLV.SPECIAL);
	}

	public <T> Optional<T> playSpecial(OctoBitConverter<T> type) {
		Optional<ByteBuffer> data = playSpecial();
		if (data.isEmpty()) {
			return Optional.empty();
		}
		return Optional.of(type.fromBytes(data.get().array()));
	}

	public List<String> playRelays() {
		List<String> result = new ArrayList<>();
		for (OctoBitChainFrame frame : chain.getFramesByType(NoStrImplShareTLV.RELAY)) {
			result.add(OctoBitFormat.IDN.fromBytes(frame.getData()));
		}
		return result;
	}

	public Optional<NoStrIdentityPublicKey> playAuthor() {
		byte[] data = chain.getFirstDataOfType(NoStrImplShareTLV.AUTHOR);
		if (data == null) {
			return Optional.empty();
		}
		return Optional.of(new NoStrIdentityPublicKey(data));
	}

	public Optional<NoStrEventKind> playKind() {
		byte[] data = chain.getFirstDataOfType(NoStrImplShareTLV.KIND);
		if (data == null) {
			return Optional.empty();
		}
		return Optional.of(NoStrEventKind.valueOf(OctoBitFormat.BI_UI32.fromBytes(data).intValue()));
	}

	public Optional<Byte> playByteOf(NoStrImplShareTLV tlv) {
		byte[] data = chain.getFirstDataOfType(tlv);
		if (data == null) {
			return Optional.empty();
		}
		if (data.length == 0) {
			return Optional.empty();
		}
		return Optional.of(data[0]);
	}

	public <T> Optional<T> playBitFormatOf(NoStrImplShareTLV tlv, OctoBitConverter<T> type) {
		byte[] data = chain.getFirstDataOfType(tlv);
		if (data == null) {
			return Optional.empty();
		}
		return Optional.of(type.fromBytes(data));
	}
}
