package love.distributedrebirth.no2all.nostr.model.share;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPublicKey;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventKind;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplShareTLV;
import love.distributedrebirth.no2all.octo.OctoBitChain;
import love.distributedrebirth.no2all.octo.OctoBitConverter;
import love.distributedrebirth.no2all.octo.OctoBitFormat;

public class NoStrShareBitChainTapeStudio extends NoStrShareBitChainTapeDeck {

	public NoStrShareBitChainTapeStudio(OctoBitChain chain) {
		super(chain);
	}

	public void clear() {
		chain.clear();
	}

	public void recordClient(String client) {
		if (client == null) {
			return;
		}
		chain.addFrame(NoStrImplShareTLV.CLIENT, OctoBitFormat.UTF8.toBytes(client));
	}

	public void recordByteArrayFor(byte[] value, NoStrImplShareTLV tlv) {
		if (value == null) {
			return;
		}
		chain.addFrame(tlv, value);
	}

	public void recordSpecial(byte[] value) {
		recordByteArrayFor(Objects.requireNonNull(value), NoStrImplShareTLV.SPECIAL);
	}

	public <T> void recordSpecial(T value, OctoBitConverter<T> type) {
		recordSpecial(type.toBytes(Objects.requireNonNull(value)));
	}

	public void recordRelays(List<String> relays) {
		if (relays == null) {
			return;
		}
		for (String relay : relays) {
			chain.addFrame(NoStrImplShareTLV.RELAY, OctoBitFormat.IDN.toBytes(relay));
		}
	}

	public void recordAuthor(Optional<NoStrIdentityPublicKey> value) {
		if (value.isEmpty()) {
			return;
		}
		recordAuthor(value.get());
	}

	public void recordAuthor(NoStrIdentityPublicKey value) {
		recordByteArrayFor(value.getBytes(), NoStrImplShareTLV.AUTHOR);
	}

	public void recordKind(Optional<NoStrEventKind> value) {
		if (value.isEmpty()) {
			return;
		}
		recordKind(value.get());
	}

	public void recordKind(NoStrEventKind value) {
		if (value == null) {
			return;
		}
		chain.addFrame(NoStrImplShareTLV.KIND, OctoBitFormat.BI_UI32.toBytes((long)value.getNumber()));
	}

	public void recordByteFor(byte value, NoStrImplShareTLV tlv) {
		byte[] data = new byte[1];
		data[0] = value;
		chain.addFrame(tlv, data);
	}
}
