package love.distributedrebirth.no2all.nostr.model.share;

import love.distributedrebirth.no2all.nostr.nip.NoStrImplShareLink;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplShareTLV;
import love.distributedrebirth.no2all.octo.OctoBech32String;

public final class NoStrShareLinkPrivateKeySecure extends NoStrShareB32LinkTLV {

	public NoStrShareLinkPrivateKeySecure(OctoBech32String link) {
		super(NoStrImplShareLink.PRIVATE_KEY_SECURE, link);
	}

	public NoStrShareLinkPrivateKeySecure(byte[] cryptKey, byte version, byte logN, byte[] salt, byte[] nonce, byte level) {
		super(NoStrImplShareLink.PRIVATE_KEY_SECURE, tapeStudio -> {
			tapeStudio.recordSpecial(cryptKey);
			tapeStudio.recordByteFor(version, NoStrImplShareTLV.VERSION);
			tapeStudio.recordByteFor(logN, NoStrImplShareTLV.LOG_N);
			tapeStudio.recordByteArrayFor(salt, NoStrImplShareTLV.SALT);
			tapeStudio.recordByteArrayFor(nonce, NoStrImplShareTLV.NONCE);
			tapeStudio.recordByteFor(level, NoStrImplShareTLV.LEVEL);
		});
	}

	public byte[] getCryptKey() {
		return getTapeDeck().playSpecial().get().array();
	}

	public byte getVersion() {
		return getTapeDeck().playByteOf(NoStrImplShareTLV.VERSION).get();
	}

	public byte getLogN() {
		return getTapeDeck().playByteOf(NoStrImplShareTLV.LOG_N).get();
	}

	public byte[] getSalt() {
		return getTapeDeck().playByteBufferOf(NoStrImplShareTLV.SALT).get().array();
	}

	public byte[] getNonce() {
		return getTapeDeck().playByteBufferOf(NoStrImplShareTLV.NONCE).get().array();
	}

	public byte getSecurityLevel() {
		return getTapeDeck().playByteOf(NoStrImplShareTLV.LEVEL).get();
	}

	public static NoStrShareLinkPrivateKeySecure ofB32(String linkB32) {
		return new NoStrShareLinkPrivateKeySecure(OctoBech32String.of(linkB32));
	}
}
