package love.distributedrebirth.no2all.nostr.model.tag;

import java.util.Optional;

import love.distributedrebirth.no2all.nostr.fire.NoStrFireStackFishWelder;
import love.distributedrebirth.no2all.nostr.fire.NoStrFireStackWelder;
import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPublicKey;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventKind;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventTagBase;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventTag;
import love.distributedrebirth.no2all.nostr.nip.tag.NoStrImplTagA;

public final class NoStrTagA extends NoStrEventTagBase<NoStrImplTagA> {

	public NoStrTagA(String address, String relayUrl, Optional<String> platform) {
		super(NoStrImplEventTag.A);
		setMetaArgumentStr(NoStrImplTagA.ADDRESS, address);
		setMetaArgumentStr(NoStrImplTagA.RELAY_URL, relayUrl);
		setMetaArgumentOptStr(NoStrImplTagA.PLATFORM, platform);
	}

	public NoStrTagA(String[] args) {
		super(NoStrImplEventTag.A, args);
	}

	public String getAddress() {
		return getMetaArgument(NoStrImplTagA.ADDRESS).get();
	}

	public String getRelayUrl() {
		return getMetaArgument(NoStrImplTagA.RELAY_URL).get();
	}

	public Optional<String> getPlatform() {
		return getMetaArgument(NoStrImplTagA.PLATFORM);
	}

	public NoStrEventKind getAddressKind() {
		return NoStrEventKind.valueOf(NoStrImplTagA.splitToKind(getAddress()));
	}

	public NoStrIdentityPublicKey getAddressPublicKey() {
		return NoStrIdentityPublicKey.ofHex(NoStrImplTagA.splitToPublicKey(getAddress()));
	}

	public String getAddressParameter() {
		return NoStrImplTagA.splitToParameter(getAddress());
	}

	public interface FishWelder extends NoStrFireStackFishWelder {

		default NoStrFireStackWelder tagA(String address, String relayUrl, Optional<String> platform) {
			return tag(new NoStrTagA(address, relayUrl, platform));
		}

		default NoStrFireStackWelder tagA(String address, String relayUrl, String platform) {
			return tagA(address, relayUrl, Optional.of(platform));
		}

		default NoStrFireStackWelder tagA(String address, String relayUrl) {
			return tagA(address, relayUrl, Optional.empty());
		}

		default NoStrFireStackWelder tagA(NoStrEventKind kind, NoStrIdentityPublicKey publicKey, String parameter, String relayUrl, Optional<String> platform) {
			return tagA(kind.getNumber() + NoStrImplTagA.SEPERATOR + publicKey.getHex() + NoStrImplTagA.SEPERATOR + parameter, relayUrl, platform);
		}

		default NoStrFireStackWelder tagA(NoStrEventKind kind, NoStrIdentityPublicKey publicKey, String parameter, String relayUrl, String platform) {
			return tagA(kind, publicKey, parameter, relayUrl, Optional.of(platform));
		}

		default NoStrFireStackWelder tagA(NoStrEventKind kind, NoStrIdentityPublicKey publicKey, String parameter, String relayUrl) {
			return tagA(kind, publicKey, parameter, relayUrl, Optional.empty());
		}
	}
}
