package love.distributedrebirth.no2all.nostr.nip;

import java.util.Arrays;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

public enum NoStrImpl implements NoStrImplPossibility {

	NIP_0_UNKNOWN,
	NIP_1_BASE_PROTOCOL,
	NIP_2_CONTACTS,
	NIP_3_OPEN_TIMESTAMPS,
	NIP_4_CRYTE_DIRECT_MSG,
	NIP_5_NOSTR_TO_DNS,
	NIP_6_KEY_FROM_MNEMONIC_SEED,
	NIP_7_BROWSER_WINDOW_NOSTR,
	NIP_9_EVENT_DELETION,
	NIP_10_CORRECT_BASE_TAGS,
	NIP_11_RELAY_INFO_DOC,
	NIP_12_GENERIC_TAG_QUERY,
	NIP_13_PROOF_OF_WORK,
	NIP_14_TEXT_SUBJECT,
	NIP_15_MARKETPLACE,
	NIP_16_EVENT_TREATMENT,
	NIP_18_REPOSTS,
	NIP_19_BECH32_ENCODED,
	NIP_20_COMMAND_RESULT,
	NIP_21_NOSTR_URL_SCHEMA,
	NIP_22_EVENT_AT_LIMITS,
	NIP_23_LONG_FORM_CONTENT,
	NIP_25_REACTIONS,
	NIP_26_DELEGATED_EVENT_SIGN,
	NIP_27_TEXT_NOTE_REFERENCE,
	NIP_28_PUBLIC_CHAT,
	NIP_30_CUSTOM_EMOJI,
	NIP_31_ALT_FOR_UNKNOWN,
	NIP_33_PARA_REPLACEABLE_EVENTS,
	NIP_36_SENSITIVE_CONTENT,
	NIP_39_EXT_INDENTITIES_IN_PROFILE,
	NIP_40_EXPIRATION_TIMESTAMP,
	NIP_42_AUTH_RELAY,
	NIP_45_COUNTING_RESULTS,
	NIP_46_NOSTR_CONNECT,
	NIP_47_WALLET_CONNECT,
	NIP_50_KEYWORD_FILTER,
	NIP_51_LISTS,
	NIP_56_REPORTING,
	NIP_57_LIGHTING_ZAPS,
	NIP_58_BADGES,
	NIP_65_RELAY_LIST_METADATA,
	NIP_78_APP_CUSTOM_DATA,
	@Deprecated
	NIP_89_RECOMMENDED_APP_HANDLERS, // used dyna hash tags so client will a cancer disconnect if a mongool send this packet.
	NIP_94_FILE_METADATA,
	;

	private static final char PREFIX_SPLIT = '_';
	private static final int PREFIX_LENGTH = 4;
	private static final Map<Integer, NoStrImpl> MAP_NUMBER = Collections.unmodifiableMap(
			Arrays.asList(values()).stream().collect(Collectors.toMap(v -> v.getNumber(), v -> v)));
	private final int number;
	private final String qFlag;

	private NoStrImpl() {
		int secondSplitIdx = name().indexOf(PREFIX_SPLIT, PREFIX_LENGTH);
		String numberStr = name().substring(PREFIX_LENGTH, secondSplitIdx);
		this.number = Integer.valueOf(numberStr);
		this.qFlag = name();
	}

	@Override
	public int getNumber() {
		return number;
	}

	@Override
	public String getQFlag() {
		return qFlag;
	}

	public static boolean valueOfNIPValid(int nip) {
		return MAP_NUMBER.containsKey(nip);
	}

	public static NoStrImpl valueOfNIP(int nip) {
		return MAP_NUMBER.get(nip);
	}
}
