package love.distributedrebirth.no2all.nostr.nip;

import java.util.List;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonArrayBuilder;
import jakarta.json.JsonObject;
import jakarta.json.JsonObjectBuilder;
import jakarta.json.JsonString;
import jakarta.json.JsonValue;
import love.distributedrebirth.no2all.nostr.NoStrGospel;

public final class NoStrImplChuch {

	public static void readArrayString(JsonObject json, NoStrImplQName jsonName, List<String> data) {
		readArrayString(json, jsonName.getQName(), data);
	}
	
	public static void readArrayString(JsonObject json, String jsonName, List<String> data) {
		readArray(json, jsonName, v -> data.add(((JsonString)v).getString()));
	}

	public static void readArray(JsonObject json, NoStrImplQName jsonName, Consumer<JsonValue> arrayValue) {
		readArray(json, jsonName.getQName(), arrayValue);
	}

	public static void readArray(JsonObject json, String jsonName, Consumer<JsonValue> arrayValue) {
		if (!json.containsKey(jsonName)) {
			return;
		}
		JsonArray values = json.getJsonArray(jsonName);
		for (JsonValue valueJson : values) {
			arrayValue.accept(valueJson);
		}
	}

	public static String readString(JsonObject json, NoStrImplQName jsonName) {
		return readString(json, jsonName.getQName());
	}

	public static String readString(JsonObject json, String jsonName) {
		return json.getString(jsonName, null);
	}

	public static Integer readInteger(JsonObject json, NoStrImplQName jsonName) {
		return readInteger(json, jsonName.getQName());
	}

	public static Integer readInteger(JsonObject json, String jsonName) {
		if (json.containsKey(jsonName)) {
			return json.getInt(jsonName);
		}
		return null;
	}

	public static Boolean readBoolean(JsonObject json, NoStrImplQName jsonName) {
		return readBoolean(json, jsonName.getQName());
	}

	public static Boolean readBoolean(JsonObject json, String jsonName) {
		if (json.containsKey(jsonName)) {
			return json.getBoolean(jsonName);
		}
		return null;
	}

	public static void writeArrayWriter(JsonObjectBuilder json, NoStrImplQName jsonName, List<? extends NoStrGospel> data) {
		writeArrayWriter(json, jsonName.getQName(), data);
	}

	public static void writeArrayWriter(JsonObjectBuilder json, String jsonName, List<? extends NoStrGospel> data) {
		writeArray(json, jsonName, data, (r, v) -> r.add(v.toBible()));
	}

	public static void writeArrayString(JsonObjectBuilder json, NoStrImplQName jsonName, List<String> data) {
		writeArrayString(json, jsonName.getQName(), data);
	}

	public static void writeArrayString(JsonObjectBuilder json, String jsonName, List<String> data) {
		writeArray(json, jsonName, data, (r, v) -> r.add(v));
	}

	public static <T> void writeArray(JsonObjectBuilder json, NoStrImplQName jsonName, List<T> data, BiConsumer<JsonArrayBuilder, T> addFn) {
		writeArray(json, jsonName.getQName(), data, addFn);
	}

	public static <T> void writeArray(JsonObjectBuilder json, String jsonName, List<T> data, BiConsumer<JsonArrayBuilder, T> addFn) {
		JsonArrayBuilder builder = Json.createArrayBuilder();
		for (T value : data) {
			addFn.accept(builder, value);
		}
		JsonArray result = builder.build();
		if (!result.isEmpty()) {
			json.add(jsonName, result);
		}
	}

	public static void writeString(JsonObjectBuilder json, NoStrImplQName jsonName, String data) {
		writeString(json, jsonName.getQName(), data);
	}

	public static void writeString(JsonObjectBuilder json, String jsonName, String data) {
		if (data == null) {
			return;
		}
		json.add(jsonName, data);
	}

	public static void writeInteger(JsonObjectBuilder json, NoStrImplQName jsonName, Integer data) {
		writeInteger(json, jsonName.getQName(), data);
	}

	public static void writeInteger(JsonObjectBuilder json, String jsonName, Integer data) {
		if (data == null) {
			return;
		}
		json.add(jsonName, data);
	}

	public static void writeBoolean(JsonObjectBuilder json, NoStrImplQName jsonName, Boolean data) {
		writeBoolean(json, jsonName.getQName(), data);
	}

	public static void writeBoolean(JsonObjectBuilder json, String jsonName, Boolean data) {
		if (data == null) {
			return;
		}
		json.add(jsonName, data);
	}
}
