package love.distributedrebirth.no2all.nostr.nip;

import java.util.Arrays;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

import love.distributedrebirth.no2all.nostr.model.event.NoStrEventKind;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventKindRange;

public enum NoStrImplEventKind implements NoStrEventKind {

	METADATA			(0, NoStrImpl.NIP_1_BASE_PROTOCOL),
	TEXT_NOTE			(1, NoStrImpl.NIP_1_BASE_PROTOCOL),
	RECOMMEND_RELAY		(2, NoStrImpl.NIP_1_BASE_PROTOCOL),
	CONTACTS			(3, NoStrImpl.NIP_2_CONTACTS),
	CRYPT_DIRECT_MSG	(4, NoStrImpl.NIP_4_CRYTE_DIRECT_MSG),
	EVENT_DELETION		(5, NoStrImpl.NIP_9_EVENT_DELETION),
	REPOSTS				(6, NoStrImpl.NIP_18_REPOSTS),
	REACTION			(7, NoStrImpl.NIP_25_REACTIONS),
	BADGE_AWARD			(8, NoStrImpl.NIP_58_BADGES),

	CHANNEL_CREATION	(40, NoStrImpl.NIP_28_PUBLIC_CHAT),
	CHANNEL_METADATA	(41, NoStrImpl.NIP_28_PUBLIC_CHAT),
	CHANNEL_SEND_MSG	(42, NoStrImpl.NIP_28_PUBLIC_CHAT),
	CHANNEL_HIDE_MSG	(43, NoStrImpl.NIP_28_PUBLIC_CHAT),
	CHANNEL_MUTE_USER	(44, NoStrImpl.NIP_28_PUBLIC_CHAT),

	FILE_METADATA		(1063, NoStrImpl.NIP_94_FILE_METADATA),
	REPORTING			(1984, NoStrImpl.NIP_56_REPORTING),
	ZAP_REQUEST			(9734, NoStrImpl.NIP_57_LIGHTING_ZAPS),
	ZAP_TX				(9735, NoStrImpl.NIP_57_LIGHTING_ZAPS),

	MUTE_LIST			(10000, NoStrImpl.NIP_51_LISTS),
	PIN_LIST			(10001, NoStrImpl.NIP_51_LISTS),
	RELAY_LIST_METADATA	(10002, NoStrImpl.NIP_65_RELAY_LIST_METADATA),
	WALLET_INFO			(13194, NoStrImpl.NIP_47_WALLET_CONNECT),

	CLIENT_AUTH			(22242, NoStrImpl.NIP_42_AUTH_RELAY),
	WALLET_REQUEST		(23194, NoStrImpl.NIP_47_WALLET_CONNECT),
	WALLET_RESPONSE		(23195, NoStrImpl.NIP_47_WALLET_CONNECT),
	NOSTR_CONNECT		(24133, NoStrImpl.NIP_46_NOSTR_CONNECT),

	CAT_PEOPLE_LIST		(30000, NoStrImpl.NIP_51_LISTS),
	CAT_BOOKMARK_LIST	(30001, NoStrImpl.NIP_51_LISTS),
	PROFILE_BADGES		(30008, NoStrImpl.NIP_58_BADGES),
	BADGE_DEFINITION	(30009, NoStrImpl.NIP_58_BADGES),
	MARKET_STALL		(30017, NoStrImpl.NIP_15_MARKETPLACE),
	MARKET_PRODUCT		(30018, NoStrImpl.NIP_15_MARKETPLACE),
	LONG_FORM_CONTENT	(30023, NoStrImpl.NIP_23_LONG_FORM_CONTENT),
	APP_CUSTOM_DATA		(30078, NoStrImpl.NIP_78_APP_CUSTOM_DATA),
	;

	private static final Map<Integer, NoStrImplEventKind> MAP_NUMBER = Collections.unmodifiableMap(
			Arrays.asList(values()).stream().collect(Collectors.toMap(v -> v.getNumber(), v -> v)));
	private final int number;
	private final NoStrImplPossibility nip;
	private final NoStrEventKindRange range;

	private NoStrImplEventKind(int number, NoStrImpl nip) {
		this.number = number;
		this.nip = nip;
		this.range = NoStrImplEventKindRange.valueOfKind(number);
	}

	@Override
	public int getNumber() {
		return number;
	}

	@Override
	public String getQFlag() {
		return name();
	}

	@Override
	public NoStrImplPossibility getNip() {
		return nip;
	}

	@Override
	public NoStrEventKindRange getRange() {
		return range;
	}

	public static boolean valueOfKindValid(int kind) {
		return MAP_NUMBER.containsKey(kind);
	}

	public static NoStrImplEventKind valueOfKind(int kind) {
		return MAP_NUMBER.get(kind);
	}
}
