package love.distributedrebirth.no2all.nostr.nip;

import java.util.Arrays;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import jakarta.json.JsonArray;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClient;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelay;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientAuth;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientCount;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientEose;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientEvent;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientNotice;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientOk;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayAuth;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayClose;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayCount;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayEvent;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayReq;

public enum NoStrImplMessageType implements NoStrImplQName {

	EVENT	(NoStrImpl.NIP_1_BASE_PROTOCOL,		v -> new NoStrMsgToRelayEvent(v),		v -> new NoStrMsgToClientEvent(v)),
	REQ		(NoStrImpl.NIP_1_BASE_PROTOCOL,		v -> new NoStrMsgToRelayReq(v),			null),
	CLOSE	(NoStrImpl.NIP_1_BASE_PROTOCOL,		v -> new NoStrMsgToRelayClose(v),		null),
	EOSE	(NoStrImpl.NIP_1_BASE_PROTOCOL,		null,									v -> new NoStrMsgToClientEose(v)),
	NOTICE	(NoStrImpl.NIP_1_BASE_PROTOCOL,		null,									v -> new NoStrMsgToClientNotice(v)),

	OK		(NoStrImpl.NIP_20_COMMAND_RESULT,	null,									v -> new NoStrMsgToClientOk(v)),
	AUTH	(NoStrImpl.NIP_42_AUTH_RELAY,		v -> new NoStrMsgToRelayAuth(v),		v -> new NoStrMsgToClientAuth(v)),
	COUNT	(NoStrImpl.NIP_45_COUNTING_RESULTS,	v -> new NoStrMsgToRelayCount(v),		v -> new NoStrMsgToClientCount(v)),
	;

	private static final Set<String> MAP_NAME = Arrays.asList(values()).stream().map(v -> v.name()).collect(Collectors.toSet());
	private final NoStrImplPossibility requireNip;
	private final Optional<Function<JsonArray, NoStrMsgToRelay>> mapperToRelay;
	private final Optional<Function<JsonArray, NoStrMsgToClient>> mapperToClient;

	private NoStrImplMessageType(NoStrImpl requireNip,
			Function<JsonArray, NoStrMsgToRelay> mapperToRelay,
			Function<JsonArray, NoStrMsgToClient> mapperToClient) {
		this.requireNip = requireNip;
		this.mapperToRelay = Optional.ofNullable(mapperToRelay);
		this.mapperToClient = Optional.ofNullable(mapperToClient);
	}

	public NoStrImplPossibility getFeatureNip() {
		return requireNip;
	}

	public String getQName() {
		return name();
	}

	public Optional<Function<JsonArray, NoStrMsgToRelay>> getMapperToRelay() {
		return mapperToRelay;
	}

	public Optional<Function<JsonArray, NoStrMsgToClient>> getMapperToClient() {
		return mapperToClient;
	}

	public static boolean valueOfValid(String name) {
		return MAP_NAME.contains(Objects.requireNonNull(name));
	}
}
