package love.distributedrebirth.no2all.nostr.model.event;

import java.io.InputStreamReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import jakarta.json.Json;
import jakarta.json.JsonReader;
import jakarta.json.JsonWriter;
import jakarta.json.JsonWriterFactory;
import jakarta.json.stream.JsonGenerator;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventTag;

public class NoStrEventTest {

	@Test
	public void testReadEvent() throws Exception {
		try (JsonReader reader = Json.createReader(new InputStreamReader(getClass().getResourceAsStream("note-text-sign-error.json")))) {
			NoStrEvent event = new NoStrEvent(reader.readObject());
			NoStrEventPayload payload = event.getPayload();
			Assertions.assertEquals("000c5b8f354d200abd7dc0bc81bd35153aebe72824c23e834db5feb83591c20d", event.getId().getHex());
			Assertions.assertEquals(1, payload.getKind().getNumber());
			Optional<NoStrEventTag> clientTag = payload.findFirstByQName(NoStrImplEventTag.CLIENT);
			Assertions.assertTrue(clientTag.isPresent());
			Assertions.assertEquals("more-speech", clientTag.get().getMetaArgument(0).get().split(" ")[0]);
			Assertions.assertEquals(13, payload.findByQName(NoStrImplEventTag.P).size());
			Optional<NoStrEventTag> eTag = payload.findFirstByQName(NoStrImplEventTag.E);
			Assertions.assertTrue(eTag.isPresent());
			Assertions.assertEquals("d54e918e87b55253315df49cc44e60ca551b9124c2a3dc1b4d68c57e13ba8422", eTag.get().getMetaArgument(0).get());
			Assertions.assertEquals("reply", eTag.get().getMetaArgument(2).get());
		}
	}

	@Test
	public void testReadEventPritty() throws Exception {
		Map<String, Object> config = new HashMap<>();
		config.put(JsonGenerator.PRETTY_PRINTING, true);
		JsonWriterFactory prittyFactory = Json.createWriterFactory(config);
		try (JsonReader reader = Json.createReader(new InputStreamReader(getClass().getResourceAsStream("note-text-sign-error.json")))) {
			NoStrEvent event = new NoStrEvent(reader.readObject());
			StringWriter jsonStr = new StringWriter();
			try (JsonWriter writer = prittyFactory.createWriter(jsonStr)) {
				writer.write(event.toBible());
			}
			System.out.println(jsonStr.getBuffer());
		}
	}

	@Test
	public void testEmptyTags() throws Exception {
		try (JsonReader reader = Json.createReader(new InputStreamReader(getClass().getResourceAsStream("note-empty-tags.json")))) {
			NoStrEvent event = new NoStrEvent(reader.readObject());
			NoStrEventPayload payload = event.getPayload();
			Assertions.assertEquals("fc7f200c5bed175702bd06c7ca5dba90d3497e827350b42fc99c3a4fa276a712", event.getId().getHex());
			Assertions.assertEquals(1, payload.getKind().getNumber());
			Assertions.assertTrue(payload.getTags().isEmpty());
		}
	}
}
