package love.distributedrebirth.no2all.nostr.model.share;

import java.util.Base64;
import java.util.List;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPublicKey;
import love.distributedrebirth.no2all.octo.OctoBitFormat;

public class NoStrShareLinkProfileTest {

	@Test
	public void testToObject() {
		NoStrShareLinkProfile profile = NoStrShareLinkProfile.ofB32("nprofile1qqsrhuxx8l9ex335q7he0f09aej04zpazpl0ne2cgukyawd24mayt8gpp4mhxue69uhhytnc9e3k7mgpz4mhxue69uhkg6nzv9ejuumpv34kytnrdaksjlyr9p");
		Assertions.assertEquals("3bf0c63fcb93463407af97a5e5ee64fa883d107ef9e558472c4eb9aaaefa459d", profile.getPublicKey().getHex());
		Assertions.assertEquals(2, profile.getRelays().size());
		Assertions.assertEquals("wss://r.x.com", profile.getRelays().get(0));
		Assertions.assertEquals("wss://djbas.sadkb.com", profile.getRelays().get(1));
	}

	@Test
	public void testToString() {
		List<String> relays = List.of("wss://r.x.com", "wss://djbas.sadkb.com");
		NoStrShareLinkProfile profile = new NoStrShareLinkProfile(NoStrIdentityPublicKey.ofHex("3bf0c63fcb93463407af97a5e5ee64fa883d107ef9e558472c4eb9aaaefa459d"), relays);
		Assertions.assertEquals("nprofile1qqsrhuxx8l9ex335q7he0f09aej04zpazpl0ne2cgukyawd24mayt8gpp4mhxue69uhhytnc9e3k7mgpz4mhxue69uhkg6nzv9ejuumpv34kytnrdaksjlyr9p", profile.toShareLink());
	}

	@Test
	public void testRelayUrls() {
		//https://en.wikipedia.org/wiki/Internationalized_domain_name
		List<String> relays = List.of("wss://仙上主天.中国", "wss://ουτοπία.δπθ.gr", "wss://ԝԝԝ.facebook.com", "wss://😉.tld");
		NoStrShareLinkProfile profile = new NoStrShareLinkProfile(NoStrIdentityPublicKey.ofHex("3bf0c63fcb93463407af97a5e5ee64fa883d107ef9e558472c4eb9aaaefa459d"), relays);
		
		byte[] data = profile.getTapeDeck().chain.toByteArray();
		String b64 = Base64.getEncoder().encodeToString(data);
		System.out.println("hex_h16="+OctoBitFormat.HEX.fromBytes(data));
		System.out.println("hex_ccp="+OctoBitFormat.HEX_CCP.fromBytes(data));
		System.out.println("hex_lrp="+OctoBitFormat.HEX_DIPAVALI.fromBytes(data));
		System.out.println("b64="+b64);
		System.out.println("b32="+profile.toShareLink());
		System.out.println("----------");
		byte[] linkData = profile.toShareLink().getBytes();
		System.out.println("ccp="+OctoBitFormat.HEX_CCP.fromBytes(linkData));
		System.out.println("c6p="+Base64.getEncoder().encodeToString(linkData));
		
		NoStrShareLinkProfile profileWire = NoStrShareLinkProfile.ofB32(profile.toShareLink());
		Assertions.assertEquals(relays.get(0), profileWire.getRelays().get(0));
		Assertions.assertEquals(relays.get(1), profileWire.getRelays().get(1));
		Assertions.assertEquals(relays.get(2), profileWire.getRelays().get(2));
		Assertions.assertEquals(relays.get(3), profileWire.getRelays().get(3));
	}
}
