package love.distributedrebirth.no2all.octo;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public final class OctoBitChain {

	private static final int INDEX_OF_FIRST = 0;
	private final List<OctoBitChainFrame> frames = new ArrayList<>();

	public OctoBitChain() {
	}

	public OctoBitChain(byte[] data) {
		this(ByteBuffer.wrap(data));
	}

	public OctoBitChain(ByteBuffer buffer) {
		while (buffer.hasRemaining()) {
			byte frameType = buffer.get();
			int frameLength = buffer.get();
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			for (int i = 0; i < frameLength; i++) {
				baos.write(buffer.get());
			}
			frames.add(new OctoBitChainFrame(frameType, baos.toByteArray()));
		}
	}

	public void clear() {
		frames.clear();
	}

	public int getFrameCount() {
		return frames.size();
	}

	public void addFrame(byte type, byte[] data) {
		frames.add(new OctoBitChainFrame(type, Objects.requireNonNull(data)));
	}
	
	public void addFrame(OctoBitChainFrameType type, byte[] data) {
		addFrame((byte) Objects.requireNonNull(type).getTypeByte(), Objects.requireNonNull(data));
	}

	public List<OctoBitChainFrame> getFramesByType(OctoBitChainFrameType type) {
		return getFramesByType((byte)Objects.requireNonNull(type).getTypeByte());
	}

	public List<OctoBitChainFrame> getFramesByType(byte type) {
		List<OctoBitChainFrame> result = new ArrayList<>();
		int framesLength = frames.size();
		for (int frameIdx = 0; frameIdx < framesLength; frameIdx++) {
			OctoBitChainFrame frame = frames.get(frameIdx);
			if (frame.getType() == type) {
				result.add(frame);
			}
		}
		return result;
	}

	public byte[] getFirstDataOfType(OctoBitChainFrameType type) {
		return getFirstDataOfType((byte)Objects.requireNonNull(type).getTypeByte());
	}

	public byte[] getFirstDataOfType(byte type) {
		List<OctoBitChainFrame> frames = getFramesByType(type);
		if (frames.isEmpty()) {
			return null;
		}
		return frames.get(INDEX_OF_FIRST).getData();
	}

	public void writeOutputStream(OutputStream out) throws IOException {
		int framesLength = frames.size();
		for (int frameIdx = 0; frameIdx < framesLength; frameIdx++) {
			OctoBitChainFrame frame = frames.get(frameIdx);
			byte[] data = frame.getData();
			out.write(frame.getType());
			out.write(data.length);
			for (int dataIdx = 0; dataIdx < data.length; dataIdx++) {
				out.write(data[dataIdx]);
			}
		}
	}

	public byte[] toByteArray() {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			writeOutputStream(baos);
		} catch (IOException ignoredOnBAOS) {
		}
		return baos.toByteArray();
	}
}
