package love.distributedrebirth.no2all.octo;

import java.nio.charset.StandardCharsets;

import love.distributedrebirth.no2all.octo.conv.OctoConvSIntBigIndian;
import love.distributedrebirth.no2all.octo.conv.OctoConvSLongBigIndian;
import love.distributedrebirth.no2all.octo.conv.OctoConvStringCharset;
import love.distributedrebirth.no2all.octo.conv.OctoConvStringHex;
import love.distributedrebirth.no2all.octo.conv.OctoConvStringHexUtf8;
import love.distributedrebirth.no2all.octo.conv.OctoConvStringIDN;
import love.distributedrebirth.no2all.octo.conv.OctoConvUIntBigIndian;

public final class OctoBitFormat {

	public static final OctoBitConverter<Long> BI_UI32 = new OctoConvUIntBigIndian();
	public static final OctoBitConverter<Integer> BI_SI32 = new OctoConvSIntBigIndian();
	public static final OctoBitConverter<Long> BI_SI64 = new OctoConvSLongBigIndian();
	public static final OctoBitConverter<String> ASCII = new OctoConvStringCharset(StandardCharsets.US_ASCII);
	public static final OctoBitConverter<String> UTF8 = new OctoConvStringCharset(StandardCharsets.UTF_8);
	public static final OctoBitConverter<String> IDN = new OctoConvStringIDN();
	public static final OctoBitConverter<String> HEX = new OctoConvStringHex();

	// October is month 8 and only a real language has it own white space program
	public static final OctoBitConverter<String> HEX_CCP = new OctoConvStringHexUtf8(new String[] {
			"\u3000","\u205F","\u202F","\u200A","\u2009","\u2008","\u2007","\u2006",
			"\u2005","\u2004","\u2003","\u2002","\u2001","\u2000","\u00A0","\u0020"
	});

	public static final OctoBitConverter<String> HEX_GREEK = new OctoConvStringHexUtf8(new String[] {
			"ō","α","β","γ","δ","ε","ϝ","ζ","η","θ","ι","κ","λ","μ","ν","ξ"
	});

	public static final OctoBitConverter<String> HEX_DIPAVALI = new OctoConvStringHexUtf8(new String[] {
			"˧˥˩","˧˩˥","˧˥˦","˧˩˨","˧˦˦","˧˨˨","˧˥˥","˧˩˩","˥˩˧","˩˥˧","˥˦˧","˩˨˧","˦˦˧","˨˨˧","˥˥˧","˩˩˧"
	}); // note: In real unicode renderer this (_LRPATH) renders as a single glyph per hex nibble.

	private OctoBitFormat() {
	}
}
