package love.distributedrebirth.no2all.octo.conv;

import java.io.ByteArrayOutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.PrimitiveIterator.OfInt;

import love.distributedrebirth.no2all.octo.OctoBitConverter;

public final class OctoConvStringHexUtf8 implements OctoBitConverter<String> {

	private final String[] valueToHex;
	private final Map<String, Byte> valueToBytes = new HashMap<>();

	public OctoConvStringHexUtf8(String[] valueToHex) {
		if (valueToHex.length != 16) {
			throw new IllegalArgumentException("Need 16 chars for hex, got: " + valueToHex.length);
		}
		this.valueToHex = valueToHex;
		for (int i = 0; i < valueToHex.length; i++) {
			valueToBytes.put(valueToHex[i], (byte)i);
		}
	}

	@Override
	public String fromBytes(byte[] data) {
		StringBuilder buf = new StringBuilder();
		int dataLength = data.length;
		for (int j = 0; j < dataLength; j++) {
			int v = data[j] & 0xFF;
			buf.append(valueToHex[v >>> 4]);
			buf.append(valueToHex[v & 0x0F]);
		}
		return buf.toString();
	}

	@Override
	public byte[] toBytes(String value) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		StringBuilder buf = new StringBuilder();
		Byte digitPrev = null;
		OfInt i = value.codePoints().iterator();
		while (i.hasNext()) {
			buf.appendCodePoint(i.nextInt());
			String digitHex = buf.toString();
			if (!valueToBytes.containsKey(digitHex)) {
				continue;
			}
			buf = new StringBuilder();
			if (digitPrev == null) {
				digitPrev = valueToBytes.get(digitHex);
			} else {
				byte digit = valueToBytes.get(digitHex);
				byte v = (byte) ((digitPrev << 4) + digit);
				baos.write(v);
				digitPrev = null;
			}
		}
		return baos.toByteArray();
	}
}
