package love.distributedrebirth.no2all.octo.trust;

import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

public final class OctoTrust {

	public static final String KEY_SPEC = "secp256k1";
	public static final int KEY_LENGTH = 32;
	private static final BigInteger MAXPRIVATEKEY = new BigInteger(
		"00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364140", 16);

	public static SecureRandom createSecureRandom() {
		try {
			return SecureRandom.getInstanceStrong();
		} catch (NoSuchAlgorithmException e) {
			throw new OctoTrustException(e);
		}
	}

	public static byte[] generateSecretKey() {
		SecureRandom secureRandom = createSecureRandom();
		byte[] privateKeyAttempt = new byte[KEY_LENGTH];
		secureRandom.nextBytes(privateKeyAttempt);
		BigInteger privateKeyCheck = new BigInteger(1, privateKeyAttempt);
		while (privateKeyCheck.compareTo(BigInteger.ZERO) == 0 || privateKeyCheck.compareTo(MAXPRIVATEKEY) == 1) {
			secureRandom.nextBytes(privateKeyAttempt);
			privateKeyCheck = new BigInteger(1, privateKeyAttempt);
		}
		return privateKeyAttempt;
	}

	public static byte[] createRandomByteArray(int length) {
		byte[] result = new byte[length];
		createSecureRandom().nextBytes(result);
		return result;
	}
}
