package love.distributedrebirth.no2all.octo.trust;

import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.SecretKeySpec;

public final class OctoTrustAES {

	private static final String CIPHER = "AES/CBC/PKCS5Padding";
	private static final String KEY_SPEC = "AES";
	private static final int IV_LENGTH = 16;
	private static final String PW_ALGORITHM = "PBKDF2WithHmacSHA256";
	private static final int PW_ITERATION_CNT = 65536;
	private static final int PW_KEY_LENGTH = 256;

	public static IvParameterSpec createRandomIv() {
		return new IvParameterSpec(OctoTrust.createRandomByteArray(IV_LENGTH));
	}

	public static boolean isSecretPasswordValid(char[] password, byte[] salt) {
		try {
			createSecretPassword(password, salt);
			return true;
		} catch (OctoTrustException e) {
			return false;
		}
	}

	public static SecretKey createSecretPassword(char[] password, byte[] salt) {
		try {
			SecretKeyFactory factory = SecretKeyFactory.getInstance(PW_ALGORITHM);
			KeySpec spec = new PBEKeySpec(password, salt, PW_ITERATION_CNT, PW_KEY_LENGTH);
			return new SecretKeySpec(factory.generateSecret(spec).getEncoded(), KEY_SPEC);
		} catch (NoSuchAlgorithmException | InvalidKeySpecException e) {
			throw new OctoTrustException(e);
		}
	}

	public static byte[] encrypt(byte[] data, SecretKey key, IvParameterSpec iv) {
		try {
			Cipher cipher = Cipher.getInstance(CIPHER);
			cipher.init(Cipher.ENCRYPT_MODE, key, iv);
			return cipher.doFinal(data);
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException
				| InvalidAlgorithmParameterException | IllegalBlockSizeException | BadPaddingException e) {
			throw new OctoTrustException(e);
		}
	}

	public static byte[] decrypt(byte[] data, SecretKey key, IvParameterSpec iv) {
		try {
			Cipher cipher = Cipher.getInstance(CIPHER);
			cipher.init(Cipher.DECRYPT_MODE, key, iv);
			return cipher.doFinal(data);
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException
				| InvalidAlgorithmParameterException | IllegalBlockSizeException | BadPaddingException e) {
			throw new OctoTrustException(e);
		}
	}
}
