package love.distributedrebirth.no2all.octo.trust;

import org.bouncycastle.crypto.agreement.ECDHBasicAgreement;
import org.bouncycastle.crypto.params.ECDomainParameters;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.math.ec.ECPoint;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.encoders.Base64;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.math.BigInteger;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Random;

public final class OctoTrustAESCurve {

	private static final String CIPHER = "AES/CBC/PKCS5Padding";
	private static final String KEY_SPEC = "AES";
	private static final int KEY_LENGTH = 32;
	private static final int IV_LENGTH = 16;
	
	private static byte[] bytesFromBigInteger(BigInteger n) {
		byte[] b = n.toByteArray();
		if (b.length == KEY_LENGTH) {
			return b;
		} else if (b.length > KEY_LENGTH) {
			return java.util.Arrays.copyOfRange(b, b.length - KEY_LENGTH, b.length);
		} else {
			byte[] buf = new byte[KEY_LENGTH];
			System.arraycopy(b, 0, buf, buf.length - b.length, b.length);
			return buf;
		}
	}

	public static BigInteger calculateKeyAgreement(BigInteger privKey, BigInteger theirPubKey) {
		ECDomainParameters param = OctoTrustAESCurveX9EC.FACTORY.getCurveEC();
		ECPrivateKeyParameters privKeyP = new ECPrivateKeyParameters(privKey, param);
		byte[] compressed = new byte[] { 2 };
		byte[] val = Arrays.concatenate(compressed, bytesFromBigInteger(theirPubKey));
		ECPoint ecPoint = param.getCurve().decodePoint(val);
		ECPublicKeyParameters pubKeyP = new ECPublicKeyParameters(ecPoint, param);
		ECDHBasicAgreement agreement = new ECDHBasicAgreement();
		agreement.init(privKeyP);
		return agreement.calculateAgreement(pubKeyP);
	}

	public static String encrypt(BigInteger key, String msg) {
		try {
			Random r = SecureRandom.getInstanceStrong();
			byte[] iv = new byte[IV_LENGTH];
			r.nextBytes(iv);
			Cipher cipher = Cipher.getInstance(CIPHER);
			cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(bytesFromBigInteger(key), KEY_SPEC), new IvParameterSpec(iv));
			String ivBase64 = Base64.toBase64String(iv);
			byte[] encryptedMsg = cipher.doFinal(msg.getBytes());
			String encryptedMsgBase64 = Base64.toBase64String(encryptedMsg);
			return String.format("%s?iv=%s", encryptedMsgBase64, ivBase64);
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException
				| InvalidAlgorithmParameterException | IllegalBlockSizeException | BadPaddingException e) {
			throw new OctoTrustException(e);
		}
	}

	public static String decrypt(BigInteger key, String encryptedMsg) {
		try {
			int pos = encryptedMsg.indexOf("?iv=");
			String msgPart = encryptedMsg.substring(0, pos);
			String ivPart = encryptedMsg.substring(pos + 4);
			byte[] decodedMsg = Base64.decode(msgPart);
			byte[] iv = Base64.decode(ivPart);
			Cipher cipher = Cipher.getInstance(CIPHER);
			cipher.init(Cipher.DECRYPT_MODE, new SecretKeySpec(bytesFromBigInteger(key), KEY_SPEC), new IvParameterSpec(iv));
			return new String(cipher.doFinal(decodedMsg));
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException
				| InvalidAlgorithmParameterException | IllegalBlockSizeException | BadPaddingException e) {
			throw new OctoTrustException(e);
		}
	}
}
