package love.distributedrebirth.no2all.octo.trust;

import java.nio.ByteBuffer;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

public class OctoTrustChaChaPx {

	private static final String CIPHER = "ChaCha20-Poly1305";
	private static final int NONCE_LENGTH = 12;

	public static byte[] createRandomNonce() {
		return OctoTrust.createRandomByteArray(NONCE_LENGTH);
	}

	public byte[] encrypt(byte[] data, SecretKey key, byte[] nonce) {
		try {
			Cipher cipher = Cipher.getInstance(CIPHER);
			cipher.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(nonce));
			byte[] encrypted = cipher.doFinal(data);
			return ByteBuffer.allocate(encrypted.length + NONCE_LENGTH).put(encrypted).put(nonce).array();
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException
				| InvalidAlgorithmParameterException | IllegalBlockSizeException | BadPaddingException e) {
			throw new OctoTrustException(e);
		}
	}

	public byte[] decrypt(byte[] data, SecretKey key) {
		byte[] encrypted = new byte[data.length - NONCE_LENGTH];
		byte[] nonce = new byte[NONCE_LENGTH];
		ByteBuffer bb = ByteBuffer.wrap(data);
		bb.get(encrypted);
		bb.get(nonce);
		try {
			Cipher cipher = Cipher.getInstance(CIPHER);
			cipher.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(nonce));
			return cipher.doFinal(encrypted);
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException
				| InvalidAlgorithmParameterException | IllegalBlockSizeException | BadPaddingException e) {
			throw new OctoTrustException(e);
		}
	}
}
