package love.distributedrebirth.no2all.octo;

import java.nio.charset.StandardCharsets;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

public class OctoBitChainTest {

	private enum XHtml6 implements OctoBitChainFrameType {
		HEAD, BODY, SOUL;

		static String PREFIX = XHtml6.class.getSimpleName().toLowerCase();

		@Override
		public byte getTypeByte() {
			return (byte)ordinal();
		}
	}

	@Test
	public void testCustomTLV() {
		// Setup test
		OctoBitChain chain = new OctoBitChain();
		chain.addFrame(XHtml6.HEAD, "<meta name=\"foo1\" content=\"bar1\"/>".getBytes(StandardCharsets.UTF_8));
		chain.addFrame(XHtml6.HEAD, "<meta name=\"foo2\" content=\"bar2\"/>".getBytes(StandardCharsets.UTF_8));
		chain.addFrame(XHtml6.HEAD, "<meta name=\"foo3\" content=\"bar3\"/>".getBytes(StandardCharsets.UTF_8));
		chain.addFrame(XHtml6.BODY, "<body/>".getBytes(StandardCharsets.UTF_8));
		chain.addFrame(XHtml6.SOUL, "𑀳𑁂𑀮𑀺𑀉𑁄𑀤𑁄𑀭𑁂𑀡 𑀪𑀸𑀕".getBytes(StandardCharsets.UTF_8));
		// Convert over wire
		String linkB32 = OctoBech32.toBech32(XHtml6.PREFIX, chain.toByteArray()).getValue();
		byte[] linkB32Data = OctoBech32.fromBech32(new OctoBech32String(linkB32)).getData();
		// Test byte parser
		OctoBitChain test = new OctoBitChain(linkB32Data);
		Assertions.assertEquals(5, test.getFrameCount());
		Assertions.assertEquals(3, test.getFramesByType(XHtml6.HEAD).size());
		Assertions.assertEquals("𑀳𑁂𑀮𑀺𑀉𑁄𑀤𑁄𑀭𑁂𑀡 𑀪𑀸𑀕", new String(test.getFirstDataOfType(XHtml6.SOUL), StandardCharsets.UTF_8));
		// Test full binary
		Assertions.assertArrayEquals(chain.toByteArray(), test.toByteArray());
	}
}
