package love.distributedrebirth.no2all.octo.trust;

import java.math.BigInteger;
import java.util.HashSet;
import java.util.Set;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import love.distributedrebirth.no2all.octo.OctoBitFormat;

public class OctaTrustAESCurveTest {

	private BigInteger createSharedKey() {
		byte[] secKey = OctoBitFormat.HEX.toBytes("0000000000000000000000000000000000000000000000000000000000002222");
		byte[] pubKey = OctoBitFormat.HEX.toBytes("2ef93f01cd2493e04235a6b87b10d3c4a74e2a7eb7c3caf168268f6af73314b5");
		return OctoTrustAESCurve.calculateKeyAgreement(new BigInteger(secKey), new BigInteger(pubKey));
	}
	
	@Test
	public void testCypherSimple() throws Exception {
		BigInteger sharedKey = createSharedKey();
		String wireMessage = OctoTrustAESCurve.encrypt(sharedKey, "hi");
		String decodedMessage = OctoTrustAESCurve.decrypt(sharedKey, wireMessage);
		Assertions.assertEquals("733642b9f465207b29ca9070b17104db5e18f6fb6de1a88ee494f00f01eaca7", sharedKey.toString(16));
		Assertions.assertEquals("hi", decodedMessage);
	}
	
	@Test
	public void testCypherRepeat() throws Exception {
		BigInteger sharedKey = createSharedKey();
		Set<String> results = new HashSet<>();
		for (int i = 0; i < 33; i++) {
			String wireMessage = OctoTrustAESCurve.encrypt(sharedKey, "foo");
			Assertions.assertFalse(results.contains(wireMessage));
			results.add(wireMessage);
		}
	}
}
