package love.distributedrebirth.no2all.react.nostr.client;

import love.distributedrebirth.no2all.nostr.NoStr;
import love.distributedrebirth.no2all.nostr.fire.NoStrFirePipe;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelay;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientAuth;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientCount;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientEose;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientEvent;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientNotice;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientOk;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayAuth;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayClose;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayCount;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayEvent;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayReq;
import love.distributedrebirth.no2all.react.No2AllReactSlotLoad;
import love.distributedrebirth.no2all.react.No2AllReact;
import love.distributedrebirth.no2all.react.No2AllReactListener;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.No2AllReactSlotSignal;
import love.distributedrebirth.no2all.react.No2AllReactTypeScript;
import love.distributedrebirth.no2all.react.wire.client.No2AllActClientClose;
import love.distributedrebirth.no2all.react.wire.client.No2AllActClientSendMessage;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClient;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClientOnMessage;
import love.distributedrebirth.no2all.react.wire.client.No2AllRtsClient;

public class NoStrRtsClient implements No2AllReactTypeScript {

	public static final No2AllReactSlot API = No2AllReactSlot.ofClass(NoStrRtsClient.class);

	@Override
	public void onEvent(No2AllReactSlotSignal<No2AllReactSlotLoad> signal) {
		No2AllReact react = signal.getReact();
		react.claim(API);
		react.registrate(No2AllRtsClient.API, No2AllArtClient.class, artClient -> {
			No2AllReactSlot clientSlot = artClient.getData().getSlot();
			react.claimIn(clientSlot, NoStrMsgToRelayAuth.class);
			react.claimIn(clientSlot, NoStrMsgToRelayClose.class);
			react.claimIn(clientSlot, NoStrMsgToRelayCount.class);
			react.claimIn(clientSlot, NoStrMsgToRelayEvent.class);
			react.claimIn(clientSlot, NoStrMsgToRelayReq.class);
			react.claimOut(clientSlot, NoStrMsgToClientAuth.class);
			react.claimOut(clientSlot, NoStrMsgToClientCount.class);
			react.claimOut(clientSlot, NoStrMsgToClientEose.class);
			react.claimOut(clientSlot, NoStrMsgToClientEvent.class);
			react.claimOut(clientSlot, NoStrMsgToClientNotice.class);
			react.claimOut(clientSlot, NoStrMsgToClientOk.class);
			NoStrFirePipe pipeReader = NoStr.FACTORY.pipeReaderClient(reason -> {
				react.fire(clientSlot, No2AllActClientClose.ofRefuse(reason));
			}, message -> {
				react.fire(clientSlot, message);
			});
			react.registrate(clientSlot, No2AllArtClientOnMessage.class, v -> {
				pipeReader.onFire(v.getData().getMessage());
			});
			react.registrate(clientSlot, NoStrMsgToRelayAuth.class, pipeWriter(react, clientSlot));
			react.registrate(clientSlot, NoStrMsgToRelayClose.class, pipeWriter(react, clientSlot));
			react.registrate(clientSlot, NoStrMsgToRelayCount.class, pipeWriter(react, clientSlot));
			react.registrate(clientSlot, NoStrMsgToRelayEvent.class, pipeWriter(react, clientSlot));
			react.registrate(clientSlot, NoStrMsgToRelayReq.class, pipeWriter(react, clientSlot));
		});
	}

	private <T extends NoStrMsgToRelay> No2AllReactListener<T> pipeWriter(No2AllReact react, No2AllReactSlot clientSlot) {
		return v -> {
			react.fire(clientSlot, new No2AllActClientSendMessage(v.getData().toBible().toString()));
		};
	}
}
