package love.distributedrebirth.no2all.react.nostr.server;

import love.distributedrebirth.no2all.nostr.NoStr;
import love.distributedrebirth.no2all.nostr.fire.NoStrFirePipe;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClient;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientAuth;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientCount;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientEose;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientEvent;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientNotice;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientOk;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayAuth;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayClose;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayCount;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayEvent;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayReq;
import love.distributedrebirth.no2all.react.No2AllReactSlotLoad;
import love.distributedrebirth.no2all.react.No2AllReact;
import love.distributedrebirth.no2all.react.No2AllReactListener;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.No2AllReactSlotSignal;
import love.distributedrebirth.no2all.react.No2AllReactTypeScript;
import love.distributedrebirth.no2all.react.wire.server.No2AllActServerBroadcastMessage;
import love.distributedrebirth.no2all.react.wire.server.No2AllActServerSocketClose;
import love.distributedrebirth.no2all.react.wire.server.No2AllActServerSocketSendMessage;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServer;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServerSocket;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServerSocketOnMessage;
import love.distributedrebirth.no2all.react.wire.server.No2AllRtsServer;

public class NoStrRtsServer implements No2AllReactTypeScript {

	public static final No2AllReactSlot API = No2AllReactSlot.ofClass(NoStrRtsServer.class);

	@Override
	public void onEvent(No2AllReactSlotSignal<No2AllReactSlotLoad> signal) {
		No2AllReact react = signal.getReact();
		react.claim(API);
		react.requireSlot(API, No2AllRtsServer.API);
		react.claimIn(API, NoStrMsgToClientNotice.class);
		react.registrate(API, NoStrMsgToClientNotice.class, v -> {
			react.fireForEach(No2AllRtsServer.API, new No2AllActServerBroadcastMessage(v.getData().toBible().toString()));
		});
		react.registrate(No2AllRtsServer.API, No2AllArtServer.class, artServer -> {
			No2AllReactSlot serverSlot = artServer.getData().getSlot();
			react.claimIn(serverSlot, NoStrMsgToClientNotice.class);
			react.registrate(serverSlot, No2AllArtServerSocket.class, artSocket -> {
				No2AllReactSlot socketSlot = artSocket.getData().getSlot();
				react.claimIn(socketSlot, NoStrMsgToClientAuth.class);
				react.claimIn(socketSlot, NoStrMsgToClientCount.class);
				react.claimIn(socketSlot, NoStrMsgToClientEose.class);
				react.claimIn(socketSlot, NoStrMsgToClientEvent.class);
				react.claimIn(socketSlot, NoStrMsgToClientNotice.class);
				react.claimIn(socketSlot, NoStrMsgToClientOk.class);
				react.claimOut(socketSlot, NoStrMsgToRelayAuth.class);
				react.claimOut(socketSlot, NoStrMsgToRelayClose.class);
				react.claimOut(socketSlot, NoStrMsgToRelayCount.class);
				react.claimOut(socketSlot, NoStrMsgToRelayEvent.class);
				react.claimOut(socketSlot, NoStrMsgToRelayReq.class);
				NoStrFirePipe pipeReader = NoStr.FACTORY.pipeReaderRelay(reason -> {
					react.fire(socketSlot, No2AllActServerSocketClose.ofRefuse(reason));
				}, message -> {
					react.fire(socketSlot, message);
				});
				react.registrate(socketSlot, No2AllArtServerSocketOnMessage.class, v -> {
					pipeReader.onFire(v.getData().getMessage());
				});
				react.registrate(socketSlot, NoStrMsgToClientAuth.class, pipeWriter(react, socketSlot));
				react.registrate(socketSlot, NoStrMsgToClientCount.class, pipeWriter(react, socketSlot));
				react.registrate(socketSlot, NoStrMsgToClientEose.class, pipeWriter(react, socketSlot));
				react.registrate(socketSlot, NoStrMsgToClientEvent.class, pipeWriter(react, socketSlot));
				react.registrate(socketSlot, NoStrMsgToClientNotice.class, pipeWriter(react, socketSlot));
				react.registrate(socketSlot, NoStrMsgToClientOk.class, pipeWriter(react, socketSlot));
			});
			react.registrate(serverSlot, NoStrMsgToClientNotice.class, v -> {
				react.fire(serverSlot, new No2AllActServerBroadcastMessage(v.getData().toBible().toString()));
			});
		});
	}

	private <T extends NoStrMsgToClient> No2AllReactListener<T> pipeWriter(No2AllReact react, No2AllReactSlot clientSlot) {
		return v -> {
			react.fire(clientSlot, new No2AllActServerSocketSendMessage(v.getData().toBible().toString()));
		};
	}
}
