package love.distributedrebirth.no2all.react.nostr.server.relay;

import java.util.logging.Logger;

import love.distributedrebirth.no2all.nostr.model.event.NoStrEvent;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEventSignature;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToClientOk;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayEvent;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplMessageOkReason;
import love.distributedrebirth.no2all.react.No2AllReact;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.No2AllReactSlotLoad;
import love.distributedrebirth.no2all.react.No2AllReactSlotSignal;
import love.distributedrebirth.no2all.react.No2AllReactTypeScript;
import love.distributedrebirth.no2all.react.nostr.server.NoStrRtsServer;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServer;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServerSocket;
import love.distributedrebirth.no2all.react.wire.server.No2AllRtsServer;

public class NoStrRtsServerRelay implements No2AllReactTypeScript {

	public static final No2AllReactSlot API = No2AllReactSlot.ofClass(NoStrRtsServerRelay.class);
	private static final Logger logger = Logger.getLogger(NoStrRtsServerRelay.class.getName());

	@Override
	public void onEvent(No2AllReactSlotSignal<No2AllReactSlotLoad> signal) {
		No2AllReact react = signal.getReact();
		react.claim(API);
		react.claimOut(API, NoStrArtServerRelayStoreEvent.class);
		react.requireSlot(API, NoStrRtsServer.API);
		react.registrate(No2AllRtsServer.API, No2AllArtServer.class, artServer -> {
			No2AllReactSlot serverSlot = artServer.getData().getSlot();
			react.registrate(serverSlot, No2AllArtServerSocket.class, artSocket -> {
				No2AllReactSlot socketSlot = artSocket.getData().getSlot();
				react.registrate(socketSlot, NoStrMsgToRelayEvent.class, v -> {
					handleRelayEvent(react, v.getData().getEvent(), artSocket.getData().getRemoteAddr(), socketSlot);
				});
			});
		});
	}

	private void handleRelayEvent(No2AllReact react, NoStrEvent event, String remoteAddr, No2AllReactSlot socketSlot) {
		boolean eventSaved = false;
		String eventResponse = "";
		NoStrImplMessageOkReason eventReason = NoStrImplMessageOkReason.DEFAULT;

		// filter on
		event.getPayload().getKind();
		event.getPayload().getPublicKey();
		event.getPayload().getTags();
		
		eventSaved = NoStrEventSignature.verify(event);
		if (!eventSaved) {
			eventReason = NoStrImplMessageOkReason.INVALID;
		}
		
		logger.fine("Validate event: " + eventSaved + " eventId: " + event.getId());
		
		
		if (eventSaved) {
			react.fire(API, new NoStrArtServerRelayStoreEvent(event, remoteAddr));
		}
		react.fire(socketSlot, new NoStrMsgToClientOk(event.getId(), eventSaved, eventReason, eventResponse));
	}
}
