package love.distributedrebirth.no2all.react.nostr.client;

import java.net.URI;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import love.distributedrebirth.no2all.nostr.NoStr;
import love.distributedrebirth.no2all.nostr.fire.NoStrFireWaterBoiler;
import love.distributedrebirth.no2all.nostr.model.NoStrIdentity;
import love.distributedrebirth.no2all.nostr.model.NoStrIdentityPrivateKey;
import love.distributedrebirth.no2all.nostr.model.event.NoStrEvent;
import love.distributedrebirth.no2all.nostr.model.message.NoStrMsgToRelayEvent;
import love.distributedrebirth.no2all.nostr.nip.NoStrImplEventKind;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.warp.No2AllReactWarpCore;
import love.distributedrebirth.no2all.react.warp.No2AllRtsWarpSpeedMonitor;
import love.distributedrebirth.no2all.react.wire.client.No2AllActClient;
import love.distributedrebirth.no2all.react.wire.client.No2AllActClientClose;
import love.distributedrebirth.no2all.react.wire.client.No2AllActClientConnect;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClient;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClientOnClose;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClientOnError;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClientOnMessage;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClientOnOpen;
import love.distributedrebirth.no2all.react.wire.client.No2AllArtClientOnStart;
import love.distributedrebirth.no2all.react.wire.client.No2AllRtsClient;
import love.distributedrebirth.no2all.wire.ojw.OjwWireClientEndpoint;

public class NoStrClientTest {

	public static void main(String[] args) throws Exception {
		ExecutorService executor = Executors.newFixedThreadPool(10);
		NoStrIdentity nid = new NoStrIdentity(NoStrIdentityPrivateKey.ofRandom());
		No2AllReactWarpCore react = new No2AllReactWarpCore();
		react.load(new No2AllRtsWarpSpeedMonitor());
		react.load(new No2AllRtsClient());
		react.load(new NoStrRtsClient());

		react.registrate(No2AllRtsClient.API, No2AllArtClient.class, v -> {
			No2AllReactSlot clientSlot = v.getData().getSlot();
			System.out.println("NoStrClient created: " + clientSlot.getSlotPath());
			react.registrate(clientSlot, No2AllArtClientOnError.class, x -> {
				System.out.println("NoStrClient socket error: " + x.getData().getError().getMessage());
			});
			react.registrate(clientSlot, No2AllArtClientOnClose.class, x -> {
				System.out.println("NoStrClient socket close: " + x.getData().getCode() + " " + x.getData().getReason());
			});
			react.registrate(clientSlot, No2AllArtClientOnOpen.class, x -> {
				System.out.println("NoStrClient socket open");
				
				executor.submit(() -> {
					try {
						for (int i = 0; i < 10; i++) {
							NoStrFireWaterBoiler userBoiler = NoStr.FACTORY.boiler(nid);
							NoStrEvent event = userBoiler.smoker()
									.tagSubject("Reply to all")
									.tagR("")
								.burn()
							.water(NoStrImplEventKind.TEXT_NOTE, "Hello world:"+i);
							
							System.out.println("send msg");
							NoStrMsgToRelayEvent msg = new NoStrMsgToRelayEvent(event);
							react.fire(clientSlot, msg);
							Thread.sleep(1000);
						}
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				});
			});
			
			react.registrate(clientSlot, No2AllArtClientOnMessage.class, y -> {
				System.out.println("NoStrClient got msg1: " + y.getData().getMessage());
			});
			// Todo "OnStart" is not working...
			react.registrate(clientSlot, No2AllArtClientOnStart.class, x -> {
				System.out.println("NoStrClient socket start");
			});
			react.fire(clientSlot, new No2AllActClientConnect());
		});
		
		OjwWireClientEndpoint clientEndpoint = new OjwWireClientEndpoint(new URI("ws://localhost:8080/event"));
		System.out.println("NoStrClient start");
		react.fire(No2AllRtsClient.API, new No2AllActClient(clientEndpoint));
		
		Thread.sleep(30000);
		
		executor.shutdown();
		executor.awaitTermination(60, TimeUnit.SECONDS);
		Thread exitTimer = new Thread(() -> {
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e2) {
				// TODO Auto-generated catch block
				e2.printStackTrace();
			}
			System.out.println("NoStrClient stopping");
			react.fireForEach(No2AllRtsClient.API, No2AllActClientClose.ofNormal("exit"));
			System.out.println("NoStrClient stopped");
		});
		exitTimer.start();
	}
}
