package love.distributedrebirth.no2all.react.wire.client;

import java.nio.ByteBuffer;

import love.distributedrebirth.no2all.react.No2AllReact;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.No2AllReactSlotLoad;
import love.distributedrebirth.no2all.react.No2AllReactSlotSignal;
import love.distributedrebirth.no2all.react.No2AllReactTypeScript;
import love.distributedrebirth.no2all.wire.WireClient;
import love.distributedrebirth.no2all.wire.WireClientEndpoint;
import love.distributedrebirth.no2all.wire.WireClientHandler;

public class No2AllRtsClient implements No2AllReactTypeScript {

	public static final No2AllReactSlot API = No2AllReactSlot.ofClass(No2AllRtsClient.class);

	@Override
	public void onEvent(No2AllReactSlotSignal<No2AllReactSlotLoad> signal) {
		No2AllReact react = signal.getReact();
		react.claim(API);
		react.claimIn(API, No2AllActClient.class);
		react.claimOut(API, No2AllArtClient.class);
		react.registrate(API, No2AllActClient.class, actClient -> {
			WireClientEndpoint clientEndpoint = actClient.getData().getEndpoint();
			No2AllReactSlot clientSlot = No2AllReactSlot.of(API, "wireId=" + clientEndpoint.wireId());
			WireClient client = clientEndpoint.wireClient(wireHandler(react, clientSlot));
			react.claim(clientSlot);
			react.claimIn(clientSlot, No2AllActClientConnect.class);
			react.claimIn(clientSlot, No2AllActClientClose.class);
			react.claimIn(clientSlot, No2AllActClientSendMessage.class);
			react.claimIn(clientSlot, No2AllActClientSendBinary.class);
			react.claimOut(clientSlot, No2AllArtClientOnStart.class);
			react.claimOut(clientSlot, No2AllArtClientOnOpen.class);
			react.claimOut(clientSlot, No2AllArtClientOnClose.class);
			react.claimOut(clientSlot, No2AllArtClientOnError.class);
			react.claimOut(clientSlot, No2AllArtClientOnMessage.class);
			react.claimOut(clientSlot, No2AllArtClientOnBinary.class);
			react.claimOut(clientSlot, WireClient.class);
			react.registrate(clientSlot, No2AllActClientConnect.class, v -> {
				client.connect();
			});
			react.registrate(clientSlot, No2AllActClientClose.class, v -> {
				client.close(v.getData().getCode(), v.getData().getMessage());
			});
			react.registrate(clientSlot, No2AllActClientSendMessage.class, v -> {
				client.sendMessage(v.getData().getMessage());
			});
			react.registrate(clientSlot, No2AllActClientSendBinary.class, v -> {
				client.sendBinary(v.getData().getMessage());
			});
			react.fire(API, new No2AllArtClient(clientSlot));
			react.fire(clientSlot, client);
		});
	}

	private WireClientHandler wireHandler(No2AllReact react, No2AllReactSlot clientSlot) {
		return new WireClientHandler() {
			@Override
			public void onStart() {
				react.fire(clientSlot, new No2AllArtClientOnStart(clientSlot));
			}

			@Override
			public void onOpen() {
				react.fire(clientSlot, new No2AllArtClientOnOpen());
			}

			@Override
			public void onClose(int code, String reason, boolean remote) {
				react.fire(clientSlot, new No2AllArtClientOnClose(code, reason, remote));
			}

			@Override
			public void onError(Throwable error) {
				react.fire(clientSlot, new No2AllArtClientOnError(error));
			}

			@Override
			public void onMessage(String message) {
				react.fire(clientSlot, new No2AllArtClientOnMessage(message));
			}

			@Override
			public void onBinary(ByteBuffer message) {
				react.fire(clientSlot, new No2AllArtClientOnBinary(message));
			}
		};
	}
}
