package love.distributedrebirth.no2all.react.wire.server;

import java.nio.ByteBuffer;

import love.distributedrebirth.no2all.react.No2AllReactSlotLoad;
import love.distributedrebirth.no2all.react.No2AllReact;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.No2AllReactSlotSignal;
import love.distributedrebirth.no2all.react.No2AllReactTypeScript;
import love.distributedrebirth.no2all.wire.WireServer;
import love.distributedrebirth.no2all.wire.WireServerEndpoint;
import love.distributedrebirth.no2all.wire.WireServerHandler;
import love.distributedrebirth.no2all.wire.WireServerSocket;

public class No2AllRtsServer implements No2AllReactTypeScript {

	public static final No2AllReactSlot API = No2AllReactSlot.ofClass(No2AllRtsServer.class);

	@Override
	public void onEvent(No2AllReactSlotSignal<No2AllReactSlotLoad> signal) {
		No2AllReact react = signal.getReact();
		react.claim(API);
		react.claimIn(API, No2AllActServer.class);
		react.claimOut(API, No2AllArtServer.class);
		react.registrate(API, No2AllActServer.class, actServer -> {
			WireServerEndpoint serverEndpoint = actServer.getData().getEndpoint();
			No2AllReactSlot serverSlot = No2AllReactSlot.of(API, "wireId=" + serverEndpoint.wireId());
			WireServer server = serverEndpoint.wireServer(wireHandler(react, serverSlot));
			react.claim(serverSlot);
			react.claimIn(serverSlot, No2AllActServerBroadcastMessage.class);
			react.claimIn(serverSlot, No2AllActServerBroadcastBinary.class);
			react.claimOut(serverSlot, No2AllArtServerOnStart.class);
			react.claimOut(serverSlot, No2AllArtServerSocket.class);
			react.claimOut(serverSlot, No2AllArtServerOnError.class);
			react.claimOut(serverSlot, WireServer.class);
			react.registrate(serverSlot, No2AllActServerBroadcastMessage.class, v -> {
				server.broadcastMessage(v.getData().getMessage());
			});
			react.registrate(serverSlot, No2AllActServerBroadcastBinary.class, v -> {
				server.broadcastBinary(v.getData().getMessage());
			});
			react.fire(API, new No2AllArtServer(serverSlot));
			react.fire(serverSlot, server);
		});
	}

	private WireServerHandler wireHandler(No2AllReact react, No2AllReactSlot serverSlot) {
		return new WireServerHandler() {
			private No2AllReactSlot socketSlug(WireServerSocket conn) {
				return No2AllReactSlot.of(serverSlot, "hashCode=" + conn.hashCode());
			}

			@Override
			public void onStart() {
				react.fire(serverSlot, new No2AllArtServerOnStart());
			}

			@Override
			public void onOpen(WireServerSocket conn) {
				No2AllReactSlot socketSlot = socketSlug(conn);
				react.claim(socketSlot);
				react.claimIn(socketSlot, No2AllActServerSocketSendMessage.class);
				react.claimIn(socketSlot, No2AllActServerSocketSendBinary.class);
				react.claimIn(socketSlot, No2AllActServerSocketClose.class);
				react.claimOut(socketSlot, No2AllArtServerSocketOnClose.class);
				react.claimOut(socketSlot, No2AllArtServerSocketOnError.class);
				react.claimOut(socketSlot, No2AllArtServerSocketOnMessage.class);
				react.claimOut(socketSlot, No2AllArtServerSocketOnBinary.class);
				react.registrate(socketSlot, No2AllActServerSocketSendMessage.class, v -> {
					conn.sendMessage(v.getData().getMessage());
				});
				react.registrate(socketSlot, No2AllActServerSocketSendBinary.class, v -> {
					conn.sendBinary(v.getData().getMessage());
				});
				react.registrate(socketSlot, No2AllActServerSocketClose.class, v -> {
					conn.close(v.getData().getCode(), v.getData().getMessage());
				});
				react.fire(serverSlot, new No2AllArtServerSocket(socketSlot, conn.getRemoteAddress()));
			}

			@Override
			public void onClose(WireServerSocket conn, int code, String reason, boolean remote) {
				No2AllReactSlot socketSlot = socketSlug(conn);
				react.fire(socketSlot, new No2AllArtServerSocketOnClose(code, reason, remote));
				react.release(socketSlot);
			}

			@Override
			public void onError(WireServerSocket conn, Throwable error) {
				if (conn != null) {
					react.fire(socketSlug(conn), new No2AllArtServerSocketOnError(error));
				} else {
					react.fire(serverSlot, new No2AllArtServerOnError(error));
				}
			}

			@Override
			public void onMessage(WireServerSocket conn, String message) {
				react.fire(socketSlug(conn), new No2AllArtServerSocketOnMessage(message));
			}

			@Override
			public void onBinary(WireServerSocket conn, ByteBuffer message) {
				react.fire(socketSlug(conn), new No2AllArtServerSocketOnBinary(message));
			}
		};
	}
}
