package love.distributedrebirth.no2all.react;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public final class No2AllReactSlot {

	private final No2AllReactSlot parent;
	private final String slotSlug;
	private final String slotPath;

	private No2AllReactSlot(No2AllReactSlot parent, String slotSlug) {
		this.parent = parent;
		this.slotSlug = slotSlug;
		this.slotPath = createSlotPath();
	}

	public No2AllReactSlot getParent() {
		return parent;
	}

	public String getSlotSlug() {
		return slotSlug;
	}

	public String getSlotPath() {
		return slotPath;
	}

	public String createSlotPath() {
		List<String> slugs = new ArrayList<>();
		No2AllReactSlot parentNode = this;
		while (parentNode != null) {
			slugs.add(parentNode.getSlotSlug());
			parentNode = parentNode.getParent();
		}
		StringBuilder buf = new StringBuilder();
		for (int i = slugs.size() - 1; i >= 0; i--) {
			buf.append(slugs.get(i));
			if (i > 0) {
				buf.append(',');
			}
		}
		return buf.toString();
	}
	
	@Override
	public int hashCode() {
		return Objects.hash(slotPath);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		No2AllReactSlot other = (No2AllReactSlot) obj;
		return Objects.equals(slotPath, other.slotPath);
	}

	public static No2AllReactSlot ofClass(Class<?> scriptClz) {
		return new No2AllReactSlot(null, scriptClz.getPackageName() + ":type=" + scriptClz.getSimpleName());
	}

	public static No2AllReactSlot of(No2AllReactSlot parent, String kvPair) {
		return new No2AllReactSlot(parent, kvPair);
	}
}
