package love.distributedrebirth.no2all.react.warp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import love.distributedrebirth.no2all.react.No2AllReactListener;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.No2AllReactSlotSignal;

public class No2AllReactWarpFuel {

	private final No2AllReactWarpCore no2AllReactWarpCore;
	private final No2AllReactSlot containerSlot;
	private final Map<Class<?>, List<No2AllReactListener<Object>>> listeners = new HashMap<>();
	private final Set<Class<?>> claimTypesOut = new HashSet<>();
	private final Set<Class<?>> claimTypesIn = new HashSet<>();
	private final Set<Class<?>> requireServices = new HashSet<>();
	private final Set<No2AllReactSlot> requireSlots = new HashSet<>();

	protected No2AllReactWarpFuel(No2AllReactWarpCore no2AllReactWarpCore, No2AllReactSlot containerSlot) {
		this.no2AllReactWarpCore = Objects.requireNonNull(no2AllReactWarpCore);
		this.containerSlot = Objects.requireNonNull(containerSlot);
	}

	private List<No2AllReactListener<Object>> toViewOfListeners(Class<?> eventType) {
		List<No2AllReactListener<Object>> result = new ArrayList<>();
		synchronized (listeners) {
			List<No2AllReactListener<Object>> typeListeners = listeners.get(eventType);
			if (typeListeners == null) {
				return result;
			}
			result.addAll(typeListeners);
		}
		return result;
	}

	public void fire(Object event, No2AllReactSlot slot) {
		for (No2AllReactListener<Object> listener : toViewOfListeners(event.getClass())) {
			listener.onEvent(new No2AllReactSlotSignal<>(slot, event, this.no2AllReactWarpCore));
		}
	}

	public void registrateListener(Class<?> eventType, No2AllReactListener<Object> listener) {
		Objects.requireNonNull(eventType);
		Objects.requireNonNull(listener);
		synchronized (listeners) {
			List<No2AllReactListener<Object>> typeListeners = listeners.get(eventType);
			if (typeListeners == null) {
				typeListeners = new ArrayList<>();
				listeners.put(eventType, typeListeners);
			}
			typeListeners.add(listener);
		}
	}

	public void registrateTypeOut(Class<?> eventType) {
		Objects.requireNonNull(eventType);
		synchronized (claimTypesOut) {
			claimTypesOut.add(eventType);
		}
	}

	public void registrateTypeIn(Class<?> eventType) {
		Objects.requireNonNull(eventType);
		synchronized (claimTypesIn) {
			claimTypesIn.add(eventType);
		}
	}

	public No2AllReactSlotContract readContract() {
		No2AllReactSlotContract result = new No2AllReactSlotContract(containerSlot);
		synchronized (claimTypesIn) {
			result.slotTypesIn.addAll(claimTypesIn);
		}
		synchronized (claimTypesOut) {
			result.slotTypesOut.addAll(claimTypesOut);
		}
		return result;
	}

	public void addRequireService(Class<?> serviceType) {
		Objects.requireNonNull(serviceType);
		synchronized (requireServices) {
			requireServices.add(serviceType);
		}
	}

	public void addRequireSlot(No2AllReactSlot dep) {
		Objects.requireNonNull(dep);
		synchronized (requireSlots) {
			requireSlots.add(dep);
		}
	}

	public void removeAll() {
		synchronized (listeners) {
			for (Class<?> eventType : listeners.keySet()) {
				List<No2AllReactListener<Object>> typeListeners = listeners.get(eventType);
				typeListeners.clear();
			}
		}
		synchronized (claimTypesOut) {
			claimTypesOut.clear();
		}
		synchronized (claimTypesIn) {
			claimTypesIn.clear();
		}
		synchronized (requireServices) {
			requireServices.clear();
		}
		synchronized (requireSlots) {
			requireSlots.clear();
		}
	}
}
