package love.distributedrebirth.no2all.wire.jetty;

import java.io.IOException;
import java.net.URI;
import java.nio.ByteBuffer;
import java.util.concurrent.ExecutionException;

import org.eclipse.jetty.websocket.client.WebSocketClient;
import org.eclipse.jetty.websocket.client.impl.JettyClientUpgradeRequest;

import love.distributedrebirth.no2all.wire.WireClient;
import love.distributedrebirth.no2all.wire.WireClientHandler;

public class JettyWireClient implements WireClient {

	private final URI uri;
	private final WebSocketClient client;
	private final JettyWireClientHandler handler;

	protected JettyWireClient(URI uri, WebSocketClient client, WireClientHandler handler) {
		this.uri = uri;
		this.client = client;
		this.handler = new JettyWireClientHandler(handler);
	}

	public JettyWireClientHandler getHandler() {
		return handler;
	}

	@Override
	public void connect() {
		try {
			client.connect(handler, uri).get();
		} catch (IOException | InterruptedException | ExecutionException e) {
			throw new IllegalStateException(e);
		}
	}

	@Override
	public void close(int code, String message) {
		client.getOpenSessions().forEach(v -> {
			JettyClientUpgradeRequest req = JettyClientUpgradeRequest.class.cast(v.getUpgradeRequest());
			if (req.getURI().equals(uri)) {
				v.close(code, message);
			}
		});
	}

	@Override
	public void sendMessage(String message) {
		client.getOpenSessions().forEach(v -> {
			try {
				v.getRemote().sendString(message);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			}
		});
	}

	@Override
	public void sendBinary(ByteBuffer message) {
		client.getOpenSessions().forEach(v -> {
			try {
				v.getRemote().sendBytes(message);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			}
		});
	}
}
