package love.distributedrebirth.no2all.wire.jetty;

import java.net.URI;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.util.component.LifeCycle;
import org.eclipse.jetty.util.component.LifeCycle.Listener;
import org.eclipse.jetty.websocket.server.config.JettyWebSocketServletContainerInitializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.warp.No2AllReactWarpCore;
import love.distributedrebirth.no2all.react.warp.No2AllRtsWarpSpeedMonitor;
import love.distributedrebirth.no2all.react.wire.server.No2AllActServer;
import love.distributedrebirth.no2all.react.wire.server.No2AllActServerSocketSendMessage;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServer;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServerSocket;
import love.distributedrebirth.no2all.react.wire.server.No2AllArtServerSocketOnMessage;
import love.distributedrebirth.no2all.react.wire.server.No2AllRtsServerLog;
import love.distributedrebirth.no2all.react.wire.server.No2AllRtsServer;

public class EventServer {

	private static final Logger LOG = LoggerFactory.getLogger(EventServer.class);

	public static void main(String[] args) throws Exception {
		EventServer server = new EventServer();
		server.setPort(8080);
		server.start();
		server.join();
	}

	private final Server server;
	private final ServerConnector connector;
	private JettyWireServerEndpoint endpoint1;
	private JettyWireServerEndpoint endpoint2;
	private JettyWireServerEndpoint endpoint3;

	public EventServer() {
		server = new Server();
		connector = new ServerConnector(server);
		server.addConnector(connector);
		ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
		context.setContextPath("/");
		server.setHandler(context);
		JettyWebSocketServletContainerInitializer.configure(context, (servletContext, wsContainer) -> {
			wsContainer.setMaxTextMessageSize(65535);
			endpoint1 = new JettyWireServerEndpoint(wsContainer, "/");
			endpoint2 = new JettyWireServerEndpoint(wsContainer, "/event");
			endpoint3 = new JettyWireServerEndpoint(wsContainer, "/other");
		});
		server.addEventListener(new Listener() {
			
			@Override
			public void lifeCycleStarted(LifeCycle event) {
				LOG.info("Startup No2AllReactWarpCore");
				No2AllReactWarpCore react = new No2AllReactWarpCore();
				react.load(new No2AllRtsWarpSpeedMonitor());
				react.load(new No2AllRtsServer());
				react.load(new No2AllRtsServerLog());
				
				// auto reply
				react.registrate(No2AllRtsServer.API, No2AllArtServer.class, artServer -> {
					No2AllReactSlot serverSlot = artServer.getData().getSlot();
					react.registrate(serverSlot, No2AllArtServerSocket.class, artSocket -> {
						No2AllReactSlot socketSlot = artSocket.getData().getSlot();
						react.registrate(socketSlot, No2AllArtServerSocketOnMessage.class, artMsg -> {
							react.fire(socketSlot, new No2AllActServerSocketSendMessage("RE: "+artMsg.getData().getMessage()));
						});
					});
				});
				react.fire(No2AllRtsServer.API, new No2AllActServer(endpoint1));
				react.fire(No2AllRtsServer.API, new No2AllActServer(endpoint2));
				react.fire(No2AllRtsServer.API, new No2AllActServer(endpoint3));
			}
		});
	}

	public void setPort(int port) {
		connector.setPort(port);
	}

	public void start() throws Exception {
		server.start();
	}

	public URI getURI() {
		return server.getURI();
	}

	public void stop() throws Exception {
		server.stop();
	}

	public void join() throws InterruptedException {
		LOG.info("Use Ctrl+C to stop server");
		server.join();
	}
}
