package love.distributedrebirth.no2all.wire.jre;

import java.net.URI;
import java.net.http.WebSocket;
import java.nio.ByteBuffer;
import java.util.concurrent.ExecutionException;

import love.distributedrebirth.no2all.wire.WireClient;
import love.distributedrebirth.no2all.wire.WireClientHandler;

public final class JreWireClient implements WireClient {

	private final URI uri;
	private final WebSocket.Builder wsBuilder;
	private final WireClientHandler handler;
	private WebSocket webSocket;

	protected JreWireClient(URI uri, WebSocket.Builder wsBuilder, WireClientHandler handler) {
		this.uri = uri;
		this.wsBuilder = wsBuilder;
		this.handler = handler;
	}

	@Override
	public void connect() {
		try {
			webSocket = wsBuilder.buildAsync(uri, new JreWireClientHandler(handler)).get();
		} catch (InterruptedException | ExecutionException error) {
			handler.onError(error); // handles: Connection refused
		}
	}

	@Override
	public void close(int code, String message) {
		if (webSocket != null) {
			webSocket.sendClose(code, message);
			webSocket = null;
		}
	}

	@Override
	public void sendMessage(String message) {
		if (webSocket != null) {
			webSocket.sendText(message, true);
		}
	}

	@Override
	public void sendBinary(ByteBuffer message) {
		if (webSocket != null) {
			webSocket.sendBinary(message, true);
		}
	}
}
