package love.distributedrebirth.no2all.wire.ojw;

import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.util.Map;
import java.util.WeakHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;

import love.distributedrebirth.no2all.wire.WireServerHandler;

public class OjwWireServerHandler extends WebSocketServer {

	private final WireServerHandler handler;
	private final Map<WebSocket, OjwWireServerSocket> wraps = new WeakHashMap<>();
	private final ReadWriteLock lock = new ReentrantReadWriteLock();
	private final Lock lockRead = lock.readLock();
	private final Lock lockWrite = lock.writeLock();

	protected OjwWireServerHandler(InetSocketAddress port, int pingTimeoutSecs, WireServerHandler handler) {
		super(port);
		setConnectionLostTimeout(pingTimeoutSecs);
		this.handler = handler;
	}

	@Override
	public void onStart() {
		handler.onStart();
	}

	@Override
	public void onOpen(WebSocket conn, ClientHandshake handshake) {
		handler.onOpen(wrap(conn));
	}

	@Override
	public void onClose(WebSocket conn, int code, String reason, boolean remote) {
		handler.onClose(wrap(conn), code, reason, remote);
	}

	@Override
	public void onError(WebSocket conn, Exception ex) {
		handler.onError(wrap(conn), ex);
	}

	@Override
	public void onMessage(WebSocket conn, String message) {
		handler.onMessage(wrap(conn), message);
	}

	@Override
	public void onMessage(WebSocket conn, ByteBuffer message) {
		handler.onBinary(wrap(conn), message);
	}

	private OjwWireServerSocket wrap(WebSocket conn) {
		OjwWireServerSocket result = null;
		lockRead.lock();
		try {
			result = wraps.get(conn);
		} finally {
			lockRead.unlock();
		}
		if (result != null) {
			return result;
		}
		result = new OjwWireServerSocket(conn);
		lockWrite.lock();
		try {
			wraps.put(conn, result);
		} finally {
			lockWrite.unlock();
		}
		return result;
	}
}