package love.distributedrebirth.no2all.zerofungus.service;

import java.io.IOException;
import java.net.URL;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;

import org.eclipse.jetty.server.CustomRequestLog;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.servlet.DefaultServlet;
import org.eclipse.jetty.servlet.ServletHolder;
import org.eclipse.jetty.util.component.LifeCycle;
import org.eclipse.jetty.util.component.LifeCycle.Listener;
import org.eclipse.jetty.util.resource.Resource;
import org.eclipse.jetty.util.resource.ResourceCollection;
import org.eclipse.jetty.webapp.Configuration;
import org.eclipse.jetty.webapp.WebAppContext;
import org.eclipse.jetty.webapp.WebInfConfiguration;
import org.eclipse.jetty.websocket.server.config.JettyWebSocketServletContainerInitializer;

import love.distributedrebirth.no2all.react.No2AllReact;
import love.distributedrebirth.no2all.react.No2AllReactSlot;
import love.distributedrebirth.no2all.react.No2AllReactSlotLoad;
import love.distributedrebirth.no2all.react.No2AllReactSlotSignal;
import love.distributedrebirth.no2all.react.No2AllReactTypeScript;
import love.distributedrebirth.no2all.react.wire.server.No2AllActServer;
import love.distributedrebirth.no2all.react.wire.server.No2AllRtsServer;
import love.distributedrebirth.no2all.wire.jetty.JettyWireServerEndpoint;
import love.distributedrebirth.no2all.zerofungus.web.RedirectServlet;

public final class ZFunRtsConfigJetty implements No2AllReactTypeScript {

	public static final No2AllReactSlot API = No2AllReactSlot.ofClass(ZFunRtsConfigJetty.class);
	private static final Logger logger = Logger.getLogger(ZFunRtsConfigJetty.class.getName());
	private final Server server;
	private final ServerConnector connector;
	private JettyWireServerEndpoint endpoint1;
	private JettyWireServerEndpoint endpoint2;
	private JettyWireServerEndpoint endpoint3;

	public ZFunRtsConfigJetty(int port) {
		server = new Server();
		connector = new ServerConnector(server);
		connector.setPort(port);
		server.addConnector(connector);
	}

	@Override
	public void onEvent(No2AllReactSlotSignal<No2AllReactSlotLoad> signal) {
		logger.info(" ╦┌─┐╔╦╗╔╦╗┬ ┬  ╔═╗┌─┐╦ ╦╔═╗┬─┐┌┬┐");
		logger.info(" ║├┤  ║  ║ └┬┘  ╠═╝│ │║║║║╣ ├┬┘ ││");
		logger.info("╚╝└─┘ ╩  ╩  ┴   ╩  └─┘╚╩╝╚═╝┴└──┴┘");
		WebAppContext context = new WebAppContext();
		context.setContextPath("/");
		context.setBaseResource(buildBaseResource(getClass().getClassLoader()));
		context.setAttribute("org.eclipse.jetty.server.webapp.ContainerIncludeJarPattern", ".*/target/classes/|.*\\.jar");
		context.setConfigurations(new Configuration[]{ 
				//new AnnotationConfiguration(),
				new WebInfConfiguration(), 
		});
		server.setHandler(context);
		
		JettyWebSocketServletContainerInitializer.configure(context, (servletContext, wsContainer) -> {
			wsContainer.setMaxTextMessageSize(65535);
			endpoint1 = new JettyWireServerEndpoint(wsContainer, "/con1");
			endpoint2 = new JettyWireServerEndpoint(wsContainer, "/event");
			endpoint3 = new JettyWireServerEndpoint(wsContainer, "/other");
		});

		logger.info("Add H2 console on: /debug/jdbc/console");
		context.addServlet("org.h2.server.web.WebServlet", "/debug/jdbc/console/*");

		context.addServlet(RedirectServlet.class, "/test");
		
		// for static
		ServletHolder staticHome = new ServletHolder("staticHome", DefaultServlet.class);
		staticHome.setInitParameter("dirAllowed","false");
		context.addServlet(staticHome, "/");
		
		logger.info("Add request logger console");
		CustomRequestLog log = new CustomRequestLog("logs/http.log", CustomRequestLog.EXTENDED_NCSA_FORMAT);
		server.setRequestLog(log);
		
		No2AllReact react = signal.getReact();
		server.addEventListener(new Listener() {
			@Override
			public void lifeCycleStarted(LifeCycle event) {
				react.fire(No2AllRtsServer.API, new No2AllActServer(endpoint1));
				react.fire(No2AllRtsServer.API, new No2AllActServer(endpoint2));
				react.fire(No2AllRtsServer.API, new No2AllActServer(endpoint3));
			}
		});
		try {
			logger.info("start jetty");
			server.start();
			logger.info("jetty started");
		} catch (Exception e) {
			throw new IllegalStateException(e);
		}
	}

	private Resource buildBaseResource(ClassLoader cl) {
		try {
			List<URL> paths = Collections.list(cl.getResources("META-INF/resources"));
			Resource[] resources = new Resource[paths.size() + 1];
			for (int i = 0; i < paths.size(); i++) {
				resources[i] = Resource.newResource(paths.get(i));
			}
			resources[paths.size()] = Resource.newResource("target/classes/static");
			return new ResourceCollection(resources);
		} catch (IOException e) {
			throw new IllegalStateException(e);
		}
	}

	public void join() {
		try {
			server.join();
		} catch (InterruptedException e) {
			throw new IllegalStateException(e);
		}
	}

	public void stop() {
		try {
			server.stop();
		} catch (Exception e) {
			throw new IllegalStateException(e);
		}
	}
}
